@@ -63,6 +63,7 @@ char *progname;
 
 #define CALC_DD		0x1
 #define CALC_DLS	0x2
+#define CALC_SLACK	0x4
 
 static void 
 usage(char *prog)
@@ -71,6 +72,7 @@ usage(char *prog)
 	printf("Slowly calculates the opposite block number\n");
 	printf("\tOne of the following must be given:\n");
 	printf("\t-d: The given address is from a 'dd' image \n");
+	printf("\t-s: The given address is from a 'dls -s' (slack) image\n");
 	printf("\t-u: The given address is from a 'dls' (unallocated) image\n");
 	printf("\t-v: verbose output to stderr\n");
 	printf("\t-V: Print version\n");
@@ -125,17 +127,97 @@ count_dls(FS_INFO *fs, DADDR_T addr, char *buf, int flags, char *ptr)
 	return WALK_CONT;
 }
 
+
+/* SLACK SPACE  call backs */
+static OFF_T flen;
+
+static u_int8_t
+count_slack_file_act(FS_INFO *fs, DADDR_T addr, char *buf, int size,
+  int flags, char *ptr)
+{
+
+	if (verbose)
+		fprintf (logfp,
+		  "count_slack_file_act: Remaining File:  %lu  Buffer: %lu\n",
+		  (ULONG)flen, (ULONG)size);
+
+	/* This is not the last data unit */
+	if (flen >= size) {
+		flen -= size;
+	}
+	/* We have passed the end of the allocated space */
+	else if (flen == 0) {
+		if (count-- == 0) {
+			printf("%lu\n", (ULONG)addr);
+			fs->close(fs);
+			exit(0);
+		}
+	}
+	/* This is the last data unit and there is unused space */
+	else if (flen < size) {
+		if (count-- == 0) {
+			printf("%lu\n", (ULONG)addr);
+			fs->close(fs);
+			exit(0);
+		}
+		flen = 0;
+	}
+
+	return WALK_CONT;
+}
+
+static u_int8_t
+count_slack_inode_act(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
+  char *ptr)
+{
+
+	if (verbose)
+		fprintf (logfp,
+		  "count_slack_inode_act: Processing meta data: %lu\n",
+		  (ULONG)inum);
+
+	/* We will now do a file walk on the content */
+	if ((fs->ftype & FSMASK) != NTFS_TYPE) {
+		flen = fs_inode->size;
+		fs->file_walk(fs, fs_inode, 0, 0,
+		  FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOABORT,
+		  count_slack_file_act, ptr);
+	}
+
+	/* For NTFS we go through each non-resident attribute */
+	else {
+		FS_DATA *fs_data = fs_inode->attr;
+
+		while ((fs_data) && (fs_data->flags & FS_DATA_INUSE)) {
+
+			if (fs_data->flags & FS_DATA_NONRES) {
+				flen = fs_data->size;
+				fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id,
+				  FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOABORT,
+				  count_slack_file_act, ptr);
+			}
+
+			fs_data = fs_data->next;
+		}
+	}
+	return WALK_CONT;
+}
+
+
+
+
+
+
 int 
 main(int argc, char **argv)
 {
-	int     flags = 0;
 	char   *fstype = DEF_FSTYPE;
 	char 	ch, type = 0;
 	FS_INFO 	*fs;
 	count = -1;
 	progname = argv[0];
 
-    while ((ch = getopt(argc, argv, "d:f:u:vV")) > 0) {
+    while ((ch = getopt(argc, argv, "d:f:s:u:vV")) > 0) {
         switch (ch) {
         case '?':
         default:
@@ -143,7 +225,6 @@ main(int argc, char **argv)
 
 		case 'd':
 			type |= CALC_DD;
-			flags = FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALIGN | FS_FLAG_DATA_META | FS_FLAG_DATA_CONT;
 			count = atoi(optarg);
 			break;
 
@@ -151,15 +232,21 @@ main(int argc, char **argv)
 			fstype = optarg;
 			break;
 
+		case 's':
+			type |= CALC_SLACK;
+			count = atoi(optarg);
+			break;
+
 		case 'u':
 			type |= CALC_DLS;
-			flags = FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALIGN | FS_FLAG_DATA_META | FS_FLAG_DATA_CONT;
 			count = atoi(optarg);
 			break; 
+
 		case 'v':
 			verbose++;
 			logfp = stderr;
 			break;
+
 		case 'V':
 			print_version();
 			exit(0);
@@ -169,7 +256,7 @@ main(int argc, char **argv)
 	if (((optind+1) != argc) || (!type) || (count < 0))
 		usage(argv[0]);
 
-	if ((type & CALC_DD) && (type & CALC_DLS)) {
+	if ( (type & CALC_DD) && (type & CALC_DLS) && (type & CALC_SLACK) ) {
 		printf("Only one block type can be given\n");
 		usage(argv[0]);
 	}
@@ -177,14 +264,28 @@ main(int argc, char **argv)
 
     fs = fs_open(argv[optind++], fstype);
 
-	if (type == CALC_DLS)
+	if (type == CALC_DLS) {
 		fs->block_walk(fs, fs->first_block, fs->last_block,
-		  flags, count_dls, (char *) fs);
-	else if (type == CALC_DD)
+		  (FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALIGN | 
+		  FS_FLAG_DATA_META | FS_FLAG_DATA_CONT),
+		  count_dls, (char *) fs);
+	}
+	else if (type == CALC_DD) {
 		fs->block_walk(fs, fs->first_block, fs->last_block,
-		  flags, count_dd, (char *) fs);
+		  (FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_UNALLOC | 
+		  FS_FLAG_DATA_ALIGN | FS_FLAG_DATA_META | FS_FLAG_DATA_CONT),
+		  count_dd, (char *) fs);
+	}
+	else if (type == CALC_SLACK) {
+		fs->inode_walk(fs, fs->first_inum, fs->last_inum,
+		  (FS_FLAG_META_ALLOC | FS_FLAG_META_USED | FS_FLAG_META_LINK),
+		  count_slack_inode_act, (char *)0);
+	}
 
     fs->close(fs);
+
+	/* We get here if the count is still > 0 */
 	printf("Block too large\n");
+
     exit(0);
 }
