@@ -15,55 +15,8 @@
 **
 */
 
-/* TCT */
-/*++
- * NAME
- *	dls 1
- * SUMMARY
- *	disk data recovery
- * SYNOPSIS
- * .ad
- * .fi
- *	\fBdls\fR [\fB-bevV\fR] [\fB-f \fIfstype\fR]
- *		\fIdevice\fR [\fIstart-stop\fR ...]
- * DESCRIPTION
- *	\fBdls\fR opens the named \fIdevice\fR and copies data blocks.
- *	By default, \fBdls\fR copies unallocated data blocks only.
+/* TCT:
  *
- * 	Arguments:
- * .IP \fB-b\fR
- *	With file systems that have logical blocks that consist of fragments,
- *	don't insert null-byte padding to preserve logical block alignment
- *	in the output.
- *	This option is a no-op with the LINUX ext2fs file system, where
- *	logical blocks and fragments have the same size.
- * .IP \fB-e\fR
- *	Copy every block. The output should be similar to dd(1).
- * .IP "\fB-f\fI fstype\fR"
- *	Specifies the file system type. The default file system type
- *	is system dependent. With most UNIX systems the default type
- *	is \fBufs\fR (Berkeley fast file system). With Linux the default
- *	type is \fBext2fs\fR (second extended file system).
- * .IP \fB-v\fR
- *	Turn on verbose mode, output to stderr.
- * .IP \fB-V\fR
- *	Turn on verbose mode, output to stdout.
- * .IP \fIdevice\fR
- *	Disk special file, or regular file containing a disk image.
- *	On UNIX systems, raw mode disk access may give better performance
- *	than block mode disk access.  LINUX disk device drivers support
- *	only block mode disk access.
- * .IP "\fIstart-stop\fR ..."
- *	Examine the specified block number or number range. Either the
- *	\fIstart\fR, the \fIstop\fR, or the \fI-stop\fR may be omitted.
- *	If \fB-b\fR is not specified, the start block must be
- *	aligned to a logical block boundary (e.g. a multiple of 8 in
- *	the case of an FFS file system). With the LINUX ext2fs file system,
- *	the start block number must be >= 1.
- * BUGS
- *	\fBdls\fR should support more file system types. Right now, support
- *	is limited to \fBext2fs\fR when built on Linux, and \fBufs\fR when
- *	built on Solaris and BSD systems.
  * LICENSE
  *	This software is distributed under the IBM Public License.
  * AUTHOR(S)
@@ -71,7 +24,7 @@
  *	IBM T.J. Watson Research
  *	P.O. Box 704
  *	Yorktown Heights, NY 10598, USA
- --*/
+ */
 
 #include "fs_tools.h"
 #include "error.h"
@@ -102,7 +55,7 @@ static void usage()
 	printf("\t-b: no block padding\n");
 	printf("\t-e: every block\n");
 	printf("\t-l: print details in time machine list format\n");
-	printf("\t-s: print slack space of FAT and NTFS images (other flags are ignored\n");
+	printf("\t-s: print slack space only (other flags are ignored\n");
 	printf("\t-v: verbose to stderr\n");
 	printf("\t-V: print version\n");
     printf("\t-f fstype: Image file system type\n");
@@ -111,6 +64,9 @@ static void usage()
 	exit(1);
 }
 
+
+
+/* call backs for listing details */
 static void
 print_list_head(FS_INFO *fs, char *image)
 {
@@ -157,6 +113,9 @@ print_list(FS_INFO *fs, DADDR_T addr, char *buf, int flags, char *ptr)
 }
 
 
+
+
+
 /* print_block - write data block to stdout */
 
 static u_int8_t
@@ -172,21 +131,41 @@ print_block(FS_INFO *fs, DADDR_T addr, char *buf, int flags, char *ptr)
 
 
 
-/* SLACK SPACE */
+
+
+/* SLACK SPACE  call backs */
 static OFF_T flen;
 
 static u_int8_t
 slack_file_act (FS_INFO *fs, DADDR_T addr, char *buf, int size,
   int flags, char *ptr)
 {
-	if (flen <= 0) 
-		fwrite(buf, size, 1, stdout);	
-	else
+
+	if (verbose)
+		fprintf (logfp,
+		  "slack_file_act: Remaining File:  %lu  Buffer: %lu\n",
+		  (ULONG)flen, (ULONG)size);
+	
+	/* This is not the last data unit */
+	if (flen >= size) {
 		flen -= size;
+	} 
+	/* We have passed the end of the allocated space */
+	else if (flen == 0) {
+		fwrite(buf, size, 1, stdout);	
+	}
+	/* This is the last data unit and there is unused space */
+	else if (flen < size) {
+		/* Clear the used space and print it */
+		memset(buf, 0, flen);
+		fwrite(buf, size, 1, stdout);	
+		flen = 0;
+	}
 
 	return WALK_CONT;
 }
 
+/* Call back for inode_walk */
 static u_int8_t
 slack_inode_act(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
   char *ptr)
@@ -197,20 +176,26 @@ slack_inode_act(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 		  "slack_inode_act: Processing meta data: %lu\n",
 		  (ULONG)inum);
 
-	flen = fs_inode->size;
-	if ((fs->ftype & FSMASK) == FATFS_TYPE) {
-		fs->file_walk(fs, fs_inode, 0, 0, FS_FLAG_FILE_SLACK,
+	/* We will now do a file walk on the content and print the
+	 * data after the specified size of the file */
+	if ((fs->ftype & FSMASK) != NTFS_TYPE) {
+		flen = fs_inode->size;
+		fs->file_walk(fs, fs_inode, 0, 0, 
+		  FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOABORT,
 		  slack_file_act, ptr);
 	}
+
+	/* For NTFS we go through each non-resident attribute */
 	else {
 		FS_DATA *fs_data = fs_inode->attr;
 
-		/* For NTFS we go through each non-resident attribute */
 		while ((fs_data) && (fs_data->flags & FS_DATA_INUSE)) {
 
 			if (fs_data->flags & FS_DATA_NONRES) {
+				flen = fs_data->size;
 				fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id, 
-				  FS_FLAG_FILE_SLACK, slack_file_act, ptr);
+				  FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOABORT, 
+				  slack_file_act, ptr);
 			}
 
 			fs_data = fs_data->next;
@@ -280,21 +265,19 @@ int     main(int argc, char **argv)
 
 
 	if (slack) {
-		if (((fs->ftype & FSMASK) != NTFS_TYPE) &&
-		  ((fs->ftype & FSMASK) != FATFS_TYPE)) {
-			printf ("Slack option only allowed with FAT and NTFS file systems\n");
-			exit (1);
-		}
+
 
 		if ((list) || (optind < argc)) {
 			fprintf(stderr, "Other options igroned with the slack space flag, try again\n");
 			exit (1);
 		}
-	
+
+		/* get the info on each allocated inode */
 		fs->inode_walk(fs, fs->first_inum, fs->last_inum, 
 		  (FS_FLAG_META_ALLOC | FS_FLAG_META_USED | FS_FLAG_META_LINK),
 		  slack_inode_act, (char *)0);
 	
+    	fs->close(fs);
 		exit(0);
 	}
 
