@@ -42,6 +42,7 @@
 #include "error.h"
 #include "ffs.h"
 #include "ext2fs.h"
+#include "fatfs.h"
 
 FILE *logfp;
 
@@ -77,8 +78,8 @@ dstat (FS_INFO *fs, DADDR_T addr, char *buf, int flags, char *ptr)
 		exit(1);
 	}
 
-	printf("%sAllocated%s\n", (flags&FS_FLAG_ALLOC)?"":"Not ",
-	  (flags&FS_FLAG_META)?" (Meta)":"");
+	printf("%sAllocated%s\n", (flags & FS_FLAG_DATA_ALLOC) ? "" : "Not ",
+	  (flags & FS_FLAG_DATA_META) ? " (Meta)" : "");
 
     if ((fs->ftype & FSMASK) == FFS_TYPE) {
         FFS_INFO *ffs = (FFS_INFO *) fs;
@@ -88,6 +89,14 @@ dstat (FS_INFO *fs, DADDR_T addr, char *buf, int flags, char *ptr)
         EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
         printf("Group: %lu\n", (ULONG)ext2fs->grpnum);
     }	
+	else if ((fs->ftype & FSMASK) == FATFS_TYPE) {
+		FATFS_INFO *fatfs = (FATFS_INFO *) fs;
+		/* Does this have a cluster address? */
+		if (addr >= fatfs->firstclustsect) {
+			printf("Cluster: %lu\n", 
+			  (ULONG) (2 + (addr - fatfs->firstclustsect) / fatfs->csize));
+		}
+	}
 
     return WALK_STOP;
 }
@@ -101,7 +110,7 @@ main(int argc, char **argv)
     extern int 	optind;
 	DADDR_T 	addr;
 	FS_INFO 	*fs;
-	int 		flags = (FS_FLAG_UNALLOC | FS_FLAG_ALLOC | FS_FLAG_META);
+	int 		flags = (FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_META | FS_FLAG_DATA_CONT);
 
     while ((ch = getopt(argc, argv, "f:uvV")) > 0) {
         switch (ch) {
@@ -133,9 +142,9 @@ main(int argc, char **argv)
 		  (ULONG)fs->last_block);
 		return 1;
 	}
-	if (addr < fs->start_block) {
+	if (addr < fs->first_block) {
 		printf("Data unit address too small for image (%lu)\n", 
-		  (ULONG)fs->start_block);
+		  (ULONG)fs->first_block);
 		return 1;
 	}
     fs->block_walk(fs, addr, addr, flags, dstat, "dstat");
