@@ -332,17 +332,17 @@ ext2fs_copy_inode(EXT2FS_INFO *ext2fs, FS_INODE *fs_inode)
      * Apply the allocated/unallocated restriction.
      */
     fs_inode->flags = (isset(imap, ext2fs->inum - ibase) ?
-           FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+           FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 
     fs_inode->flags |= (fs_inode->nlink ?
-      FS_FLAG_LINK : FS_FLAG_UNLINK);
+      FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
 
     /*
      * Apply the used/unused restriction.
      */
 
     fs_inode->flags |= (fs_inode->ctime ?
-      FS_FLAG_USED : FS_FLAG_UNUSED);
+      FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
 
 }
 
@@ -361,7 +361,12 @@ ext2fs_inode_lookup(FS_INFO *fs, INUM_T inum)
     return (fs_inode);
 }
 
-/* ext2fs_inode_walk - inode iterator */
+/* ext2fs_inode_walk - inode iterator 
+ *
+ * flags used: FS_FLAG_META_USED, FS_FLAG_META_UNUSED,
+ *  FS_FLAG_META_LINK, FS_FLAG_META_UNLINK,
+ *  FS_FLAG_META_ALLOC, FS_FLAG_META_UNALLOC
+*/
 
 void    
 ext2fs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
@@ -403,7 +408,7 @@ ext2fs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 	 * Apply the allocated/unallocated restriction.
 	 */
 	myflags = (isset(imap, inum - ibase) ?
-		   FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+		   FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 	if ((flags & myflags) != myflags)
 	    continue;
 
@@ -413,7 +418,7 @@ ext2fs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 	 * Apply the linked/unlinked restriction.
 	 */
 	myflags |= (getu16(fs, ext2fs->dinode->i_nlink) ? 
-	  FS_FLAG_LINK : FS_FLAG_UNLINK);
+	  FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
 
 	if ((flags & myflags) != myflags)
 	    continue;
@@ -423,7 +428,7 @@ ext2fs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 	 */
 
     myflags |= (getu32(fs, ext2fs->dinode->i_ctime) ? 
-	  FS_FLAG_USED : FS_FLAG_UNUSED);
+	  FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
 
 	if ((flags & myflags) != myflags)
 	    continue;
@@ -447,7 +452,11 @@ ext2fs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
     fs_inode_free(fs_inode);
 }
 
-/* ext2fs_block_walk - block iterator */
+/* ext2fs_block_walk - block iterator 
+ *
+ * flags: FS_FLAG_DATA_ALLOC, FS_FLAG_DATA_UNALLOC, FS_FLAG_DATA_CONT,
+ *  FS_FLAG_DATA_META
+*/
 
 void    
 ext2fs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
@@ -466,9 +475,9 @@ ext2fs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
     /*
      * Sanity checks.
      */
-    if (start < fs->start_block || start > fs->last_block)
+    if (start < fs->first_block || start > fs->last_block)
 		error("%s: invalid start block number: %lu", myname, (ULONG) start);
-    if (last < fs->start_block || last > fs->last_block || last < start)
+    if (last < fs->first_block || last > fs->last_block || last < start)
 		error("%s: invalid last block number: %lu", myname, (ULONG) last);
 
     /*
@@ -492,7 +501,7 @@ ext2fs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 
 	/* This is meta data that is not described in the groups */
 	if (addr < getu32(fs, ext2fs->fs->s_first_data_block)) {
-		myflags = FS_FLAG_META | FS_FLAG_ALLOC;
+		myflags = FS_FLAG_DATA_META | FS_FLAG_DATA_ALLOC;
 
 		if ((flags & myflags) == myflags) {
 			fs_read_block(fs, fs_buf, fs->block_size, addr, "data block");
@@ -541,14 +550,16 @@ ext2fs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
      * They just happen to be reserved for something else :-)
 	 */
 	myflags = (isset(bmap, addr - dbase) ?
-		   FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+		   FS_FLAG_DATA_ALLOC : FS_FLAG_DATA_UNALLOC);
     if ((addr >= dbase && addr < getu32(fs, ext2fs->group->bg_block_bitmap))
         || (addr == getu32(fs, ext2fs->group->bg_block_bitmap))
         || (addr == getu32(fs, ext2fs->group->bg_inode_bitmap))
         || (addr >= getu32(fs, ext2fs->group->bg_inode_table) && addr < dmin))
-        myflags |= FS_FLAG_META;
+        myflags |= FS_FLAG_DATA_META;
+	else 
+        myflags |= FS_FLAG_DATA_CONT;
 
-    if ((myflags & FS_FLAG_META) && (myflags & FS_FLAG_UNALLOC)) {
+    if ((myflags & FS_FLAG_DATA_META) && (myflags & FS_FLAG_DATA_UNALLOC)) {
         remark("unallocated meta block %lu!! dbase %lu dmin %lu",
            (unsigned long) addr, (unsigned long) dbase,
            (unsigned long) dmin);
@@ -587,7 +598,7 @@ ext2fs_file_walk_direct(FS_INFO *fs, FS_BUF *buf[],
     read_count = (length < buf[0]->size ? length : buf[0]->size);
 
 	if (addr > fs->last_block) {
-		if (flags & FS_FLAG_NOABORT)  {
+		if (flags & FS_FLAG_FILE_NOABORT)  {
 			if (verbose) {
 				fprintf (logfp, 
 				  "Invalid direct block address (too large): %lu",
@@ -601,30 +612,29 @@ ext2fs_file_walk_direct(FS_INFO *fs, FS_BUF *buf[],
 		}
 	}
 
-	myflags = FS_FLAG_CONT;
+	// @@@ We do not check allocation status here
+	myflags = FS_FLAG_DATA_CONT;
 
     if (addr == 0) {
-		if (flags & FS_FLAG_UNALLOC) {   
-			myflags |= FS_FLAG_UNALLOC;
+		if (0 == (flags & FS_FLAG_FILE_NOSPARSE)) {
 
-			if ((flags & FS_FLAG_AONLY) == 0) 
+			if ((flags & FS_FLAG_FILE_AONLY) == 0) 
 				memset(buf[0]->data, 0, read_count);
+
 			if (WALK_STOP == action(fs, addr, buf[0]->data, read_count, 
 			  myflags, ptr)) 
 				return 0;
 		}
     } else {
-		if (flags & FS_FLAG_ALLOC) { 
-			myflags |= FS_FLAG_ALLOC;
-
-			if ((flags & FS_FLAG_AONLY) == 0) 
-				fs_read_block(fs, buf[0], 
-				  roundup(read_count, EXT2FS_DEV_BSIZE), addr,
-					  "data block");
-			if (WALK_STOP == action(fs, addr, buf[0]->data, read_count, 
-			  myflags, ptr))
-				return 0;
+		if ((flags & FS_FLAG_FILE_AONLY) == 0) {
+			fs_read_block(fs, buf[0], 
+			  roundup(read_count, EXT2FS_DEV_BSIZE), addr,
+				  "data block");
 		}
+
+		if (WALK_STOP == action(fs, addr, buf[0]->data, read_count, 
+		  myflags, ptr))
+			return 0;
     }
     return (read_count);
 } 
@@ -644,7 +654,7 @@ ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
     	fprintf(logfp, "%s: level %d block %lu\n", myname, level, (ULONG) addr);
     
     if (addr > fs->last_block) {
-        if (flags & FS_FLAG_NOABORT)  {
+        if (flags & FS_FLAG_FILE_NOABORT)  {
 			if (verbose) {
 				fprintf (logfp, 
         		  "Invalid indirect block address (too large): %lu",
@@ -667,8 +677,8 @@ ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
     	fs_read_block(fs, buf[level], buf[level]->size, addr,
               "disk address block");
 
-	if (flags & FS_FLAG_META) {
-		int myflags = FS_FLAG_META;
+	if (flags & FS_FLAG_FILE_META) {
+		int myflags = FS_FLAG_DATA_META;
 		action(fs, addr, buf[level]->data, buf[level]->size, myflags, ptr);
 	}
 
@@ -681,7 +691,7 @@ ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
 		int prevcnt = todo_count;
 
 	    if (getu32(fs, (u_int8_t *)&iaddr[n]) > fs->last_block) {
-			if (flags & FS_FLAG_NOABORT)  {
+			if (flags & FS_FLAG_FILE_NOABORT)  {
 				if (verbose) {
 					fprintf (logfp, 
 					  "Invalid address in indirect list (too large): %lu",
@@ -713,9 +723,13 @@ ext2fs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
 
 
 /*      
+ * flag values: FS_FLAG_FILE_NOSPARSE, FS_FLAG_FILE_AONLY, FS_FLAG_FILE_SLACK
+ * FS_FLAG_FILE_META, FS_FLAG_FILE_NOABORT
  *
- * flag values: FS_FLAG_{UN}ALLOC, FS_FLAG_META, FS_FLAG_CONT, FS_FLAG_AONLY
- * FS_FLAG_SLACK
+ * nothing special is done for FS_FLAG_FILE_RECOVER
+ *
+ * The action will use the flags: FS_FLAG_DATA_CONT, FS_FLAG_DATA_META
+ * -- @@@ Currently do not do _ALLOC and _UNALLOC
  *  
  * The type and id fields are ignored with EXT2FS
  */
@@ -739,7 +753,7 @@ ext2fs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
     length = inode->size;
 
 	/* Roundup if we want the slack space on the final fragment */
-	if (flags & FS_FLAG_SLACK)
+	if (flags & FS_FLAG_FILE_SLACK)
 		length = roundup(length, fs->block_size);
 
     /*
@@ -760,7 +774,7 @@ ext2fs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
 
 	if (length > 0) {
 		for (level = 1; level <= inode->indir_count; level++)
-			buf[level] = fs_buf_alloc(fs->addr_bsize);
+			buf[level] = fs_buf_alloc(fs->file_bsize);
 
 		for (level = 1; length > 0 && level <= inode->indir_count; level++) {
 			retval = ext2fs_file_walk_indir(fs, buf, length, 
@@ -783,6 +797,11 @@ ext2fs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
 	return;
 }
 
+static void
+ext2fs_fscheck(FS_INFO *fs, FILE *hFile)
+{
+	error ("fscheck not implemented yet for EXT3FS");
+}
 
 static void
 ext2fs_fsstat(FS_INFO *fs, FILE *hFile)
@@ -918,7 +937,7 @@ ext2fs_fsstat(FS_INFO *fs, FILE *hFile)
     fprintf(hFile, "--------------------------------------------\n");
 
     fprintf(hFile, "Fragment Range: %lu - %lu\n",
-      (ULONG)fs->start_block, (ULONG)fs->last_block);
+      (ULONG)fs->first_block, (ULONG)fs->last_block);
 
     fprintf(hFile, "Block Size: %lu\n", (ULONG)fs->file_bsize);
     fprintf(hFile, "Fragment Size: %lu\n", (ULONG)fs->block_size);
@@ -1053,7 +1072,7 @@ print_addr_act (FS_INFO *fs, DADDR_T addr, char *buf,
 {  
     FILE *hFile = (FILE *)ptr;
 
-    if (flags & FS_FLAG_CONT) {
+    if (flags & FS_FLAG_DATA_CONT) {
         int i, s;
         /* cycle through the fragments if they exist */
         for (i = 0, s = size; s > 0; s-= fs->block_size, i++) {
@@ -1072,7 +1091,7 @@ print_addr_act (FS_INFO *fs, DADDR_T addr, char *buf,
     }
 
     /* this must be an indirect block pointer, so put it in the list */
-    else if (flags & FS_FLAG_META) {
+    else if (flags & FS_FLAG_DATA_META) {
         if (indir_idx < INDIR_SIZ)
             indirl[indir_idx++] = addr;
     }
@@ -1087,13 +1106,11 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     FS_INODE *fs_inode;
     char ls[12];
-    int flags = (FS_FLAG_AONLY | FS_FLAG_CONT | FS_FLAG_META |
-          FS_FLAG_ALLOC | FS_FLAG_UNALLOC );
 
     fs_inode = ext2fs_inode_lookup (fs, inum);
     fprintf(hFile, "inode: %lu\n", (ULONG) inum);
     fprintf(hFile, "%sAllocated\n",
-      (fs_inode->flags&FS_FLAG_ALLOC)?"":"Not ");
+      (fs_inode->flags & FS_FLAG_META_ALLOC)?"":"Not ");
 
     fprintf(hFile, "Group: %lu\n", (ULONG)ext2fs->grpnum);
 
@@ -1177,7 +1194,9 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
     fprintf (hFile, "\nDirect Blocks:\n");
 
     indir_idx = 0;
-    fs->file_walk(fs, fs_inode, 0, 0, flags, print_addr_act, (char *)hFile);
+    fs->file_walk(fs, fs_inode, 0, 0, 
+	  (FS_FLAG_FILE_AONLY | FS_FLAG_FILE_META),
+	  print_addr_act, (char *)hFile);
 
     if (printidx != 0)
         fprintf(hFile, "\n");
@@ -1290,17 +1309,14 @@ ext2fs_open(const char *name, unsigned char ftype)
     fs->last_inum = fs->inum_count;
     fs->first_inum = EXT2FS_FIRSTINO;
     fs->root_inum = EXT2FS_ROOTINO;
-	fs->niaddr = EXT2FS_NIADDR;
 
     fs->block_count = getu32(fs, ext2fs->fs->s_blocks_count);
-    fs->start_block = 0;
+    fs->first_block = 0;
     fs->last_block = fs->block_count - 1;
     fs->block_size =
 	fs->file_bsize =
-	fs->addr_bsize = 
 		EXT2FS_MIN_BLOCK_SIZE << getu32(fs, ext2fs->fs->s_log_block_size);
 
-    fs->block_frags = 1;
 	fs->dev_bsize = EXT2FS_DEV_BSIZE;
 
     ext2fs->group_offset = getu32(fs, ext2fs->fs->s_log_block_size) ? 
@@ -1320,6 +1336,7 @@ ext2fs_open(const char *name, unsigned char ftype)
 	fs->dent_walk = ext2fs_dent_walk;
 	fs->file_walk = ext2fs_file_walk;
 	fs->fsstat = ext2fs_fsstat;
+	fs->fscheck = ext2fs_fscheck;
 	fs->istat = ext2fs_istat;
     fs->close = ext2fs_close;
 
