@@ -338,15 +338,29 @@ dos2unixtime(u_int16_t date, u_int16_t time)
 	struct tm tm1;
 	int ret;
 
-    if (date == 0) 
-        return 0;
+	if (date == 0) 
+		return 0;
 
 	tm1.tm_sec = ((time & FATFS_SEC_MASK) >> FATFS_SEC_SHIFT) * 2;
+	if ((tm1.tm_sec < 0) || (tm1.tm_sec > 60))
+		tm1.tm_sec = 0;
+
 	tm1.tm_min = ((time & FATFS_MIN_MASK) >> FATFS_MIN_SHIFT);
+	if ((tm1.tm_min < 0) || (tm1.tm_min > 59))
+		tm1.tm_min = 0;
+
 	tm1.tm_hour = ((time & FATFS_HOUR_MASK) >> FATFS_HOUR_SHIFT);
+	if ((tm1.tm_hour < 0) || (tm1.tm_hour > 23))
+		tm1.tm_hour= 0;
 
 	tm1.tm_mday = ((date & FATFS_DAY_MASK) >> FATFS_DAY_SHIFT);
+	if ((tm1.tm_mday < 1) || (tm1.tm_mday > 31))
+		tm1.tm_mday = 0;
+
 	tm1.tm_mon = ((date & FATFS_MON_MASK) >> FATFS_MON_SHIFT) - 1;
+	if ((tm1.tm_mon < 0) || (tm1.tm_mon > 11))
+		tm1.tm_mon = 0;
+
 	tm1.tm_year = ((date & FATFS_YEAR_MASK) >> FATFS_YEAR_SHIFT) + 80;
 
 	/* set the daylight savings variable to -1 so that mktime() figures
@@ -355,8 +369,16 @@ dos2unixtime(u_int16_t date, u_int16_t time)
 
 	ret = mktime (&tm1);
 
-	if (-1 == ret) 
-		error ("dos2unixtime: Error running mktime()");
+	if (-1 == ret)  
+		error ("dos2unixtime: Error running mktime(): %d:%d:%d %d/%d/%d",
+	((time & FATFS_HOUR_MASK) >> FATFS_HOUR_SHIFT),
+	((time & FATFS_MIN_MASK) >> FATFS_MIN_SHIFT),
+	((time & FATFS_SEC_MASK) >> FATFS_SEC_SHIFT) * 2,
+	((date & FATFS_MON_MASK) >> FATFS_MON_SHIFT) - 1,
+	((date & FATFS_DAY_MASK) >> FATFS_DAY_SHIFT),
+	((date & FATFS_YEAR_MASK) >> FATFS_YEAR_SHIFT) + 80
+		  );
+	
 
 	return ret;
 }
@@ -402,7 +424,6 @@ fatfs_copy_inode(FATFS_INFO *fatfs, FS_INODE *fs_inode, DADDR_T sect)
 	fatfs_dentry    *in = fatfs->dep;
 	int 		cnum, i;
 	u_int32_t 	dcnt, clust;
-	int32_t		size;
 	FS_INFO *fs = (FS_INFO *)&fatfs->fs_info;
 
 	fs_inode->mode = dos2unixmode(in->attrib);
@@ -421,7 +442,7 @@ fatfs_copy_inode(FATFS_INFO *fatfs, FS_INODE *fs_inode, DADDR_T sect)
 	}
 	else {
 		fs_inode->nlink = (in->name[0] == FATFS_SLOT_DELETED)? 0 : 1;
-		fs_inode->size = size = getu32(fs, in->size);
+		fs_inode->size = (OFF_T) getu32(fs, in->size);
 	}
 
 
@@ -563,7 +584,7 @@ fatfs_copy_inode(FATFS_INFO *fatfs, FS_INODE *fs_inode, DADDR_T sect)
 		/* we are going to store the sectors, not clusters so calc
 		 * that value 
 		 */
-		fs_inode->size = cnum * fatfs->csize * fatfs->ssize;
+		fs_inode->size = (OFF_T)(cnum * fatfs->csize * fatfs->ssize);
 
 	}
 
