@@ -45,6 +45,8 @@
 #define FATFS_EOFS		0x0ffffff8
 #define FATFS_EOFE		0x0fffffff
 
+
+
 /* macro to identify if the FAT value is End of File
  * returns 1 if it is and 0 if it is not 
  */
@@ -52,6 +54,17 @@
 	((val >= (FATFS_EOFS & mask)) && (val <= (FATFS_EOFE)))
 
 
+#define FATFS_ISBAD(val, mask) \
+	((val) == (FATFS_BAD & mask))
+
+
+#define FATFS_CLUST_2_SECT(fatfs, c)	\
+	(fatfs->firstclustsect + ((((c) & fatfs->mask) - 2) * fatfs->csize))
+
+#define FATFS_SECT_2_CLUST(fatfs, s)	\
+	(2 + ((s)  - fatfs->firstclustsect) / fatfs->csize)
+
+
 
 /* given an inode, determine which sector it is in
  * i must be larger than 3 (2 is the root and it doesn't have a sector)
@@ -67,6 +80,7 @@
     ((s - fatfs->firstdatasect) * fatfs->dentry_cnt_se + 3)
 
 
+
 /*
  * Boot Sector Structure for FAT12, FAT16, and FAT32
  */
@@ -297,6 +311,8 @@ typedef struct {
 	/* size of data area in clusters, starting at firstdatasect */
 	u_int32_t 	clustcnt;	
 
+	u_int32_t	lastclust;
+
 	/* sector where the root directory is located */
 	u_int32_t	rootsect;
 
@@ -309,11 +325,22 @@ typedef struct {
 	u_int8_t	numfat;		/* number of fat tables */
 	u_int32_t	sectperfat;	/* sectors per fat table */
 	u_int16_t	numroot;	/* number of 32-byte dentries in root dir */
-	u_int32_t	sectors;	/* number of total sectors in image */
 	u_int32_t	mask;		/* the mask to use for the sectors */
 
 } FATFS_INFO;
 
+
+/*
+ * Macro to identify if a cluster is allocated
+ * returns 1 if it is allocated and 0 if not
+ */
+#define is_clustalloc(fs, c) \
+    (getFAT((fs), (c)) != FATFS_UNALLOC)
+
+extern u_int8_t is_sectalloc(FATFS_INFO *, int);
+
+
+
 extern void fatfs_dent_walk(FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, char *);
 extern u_int8_t fatfs_isdentry(FATFS_INFO *, fatfs_dentry *);
 extern void fatfs_make_root (FATFS_INFO *, FS_INODE *);
