@@ -5,7 +5,7 @@
 ** Human Interface Layer Support for a FFS image 
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -128,7 +128,7 @@ ffs_dent_copy(FFS_INFO *ffs, char *ffs_dent, FS_DENT *fs_dent)
 
 /* Scan the buffer for directory entries and call action on each.
 ** Flags will be
-** set to FS_FLAG_ALLOC for acive entires and FS_FLAG_UNALLOC for
+** set to FS_FLAG_NAME_ALLOC for acive entires and FS_FLAG_NAME_UNALLOC for
 ** deleted ones
 **
 ** len is size of buf
@@ -201,17 +201,17 @@ ffs_dent_parse_block (FFS_INFO *ffs, char *buf, int len,
 		myflags = 0;
 		/* Do we have a deleted entry? (are we in a deleted space) */
 		if ((dellen > 0) || (inode == 0)) {
-			myflags |= FS_FLAG_UNALLOC;
+			myflags |= FS_FLAG_NAME_UNALLOC;
 			dellen -= minreclen;
-			if (flags & FS_FLAG_UNALLOC) {
+			if (flags & FS_FLAG_NAME_UNALLOC) {
 				if (WALK_STOP == action (fs, fs_dent, myflags, ptr)) {
 					return 0;
 				}
 			}
 		} 
 		else {
-			myflags |= FS_FLAG_ALLOC;
-			if (flags & FS_FLAG_ALLOC) {
+			myflags |= FS_FLAG_NAME_ALLOC;
+			if (flags & FS_FLAG_NAME_ALLOC) {
 				if (WALK_STOP == action (fs, fs_dent, myflags, ptr)) {
 					return 0;
 				}
@@ -226,8 +226,8 @@ ffs_dent_parse_block (FFS_INFO *ffs, char *buf, int len,
 		/* if we have a directory and the RECURSE flag is set, then
 		 * lets do it
 		 */
-		if ((myflags & FS_FLAG_ALLOC) &&
-		  (flags & FS_FLAG_RECURSE)  &&
+		if ((myflags & FS_FLAG_NAME_ALLOC) &&
+		  (flags & FS_FLAG_NAME_RECURSE)  &&
 		  (!ISDOT(fs_dent->name)) &&
 		  ((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR)) {
 
@@ -279,10 +279,11 @@ ffs_dent_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 
 /* Process _inode_ as a directory inode and process the data blocks
 ** as file entries.  Call action on all entries with the flags set to
-** FS_FLAG_ALLOC for active entries
+** FS_FLAG_NAME_ALLOC for active entries
 **
 **
-** Use the following flags: FS_FLAG_ALLOC, FS_FLAG_UNALLOC, FS_FLAG_RECURSE
+** Use the following flags: FS_FLAG_NAME_ALLOC, FS_FLAG_NAME_UNALLOC, 
+** FS_FLAG_NAME_RECURSE
 */
 void 
 ffs_dent_walk(FS_INFO *fs, INUM_T inode, int flags, 
@@ -309,8 +310,7 @@ ffs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 	curdirptr = dirbuf = mymalloc(size);
 	dirleft = size;
 
-	fs->file_walk(fs, fs_inode, 0, 0, 
-	  FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK, 
+	fs->file_walk(fs, fs_inode, 0, 0, FS_FLAG_FILE_SLACK,
 	  ffs_dent_action, "");
 
 	curdirptr = NULL;
