@@ -6,7 +6,7 @@
 ** directories that exist (both active and deleted)
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carier.  All rights reserved
+** Copyright (c) 2003-2004 Brian Carier.  All rights reserved
 **
 ** TASK
 ** Copyright (c) 2002 @stake Inc.  All rights reserved
@@ -265,7 +265,7 @@ main(int argc, char **argv)
 {
 	char *fstype = DEF_FSTYPE;
 	int inode;
-	int flags = FS_FLAG_ALLOC | FS_FLAG_UNALLOC;
+	int flags = FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC;
 	char ch;
 	FS_INFO 	*fs;
 	extern int optind;
@@ -282,7 +282,7 @@ main(int argc, char **argv)
 			localFlags |= LCL_DOT;
 			break;
 		case 'd':
-			flags &= ~FS_FLAG_ALLOC;
+			flags &= ~FS_FLAG_NAME_ALLOC;
 			break;
 		case 'D':
 			localFlags &= ~LCL_FILE;
@@ -306,13 +306,13 @@ main(int argc, char **argv)
 			localFlags |= LCL_FULL;
 			break;
 		case 'r':
-			flags |= FS_FLAG_RECURSE;
+			flags |= FS_FLAG_NAME_RECURSE;
 			break;
 		case 's':
 			sec_skew = atoi(optarg);
 			break;
 		case 'u':
-			flags &= ~FS_FLAG_UNALLOC;
+			flags &= ~FS_FLAG_NAME_UNALLOC;
 			break;
 		case 'v':
 			verbose++;
@@ -345,8 +345,8 @@ main(int argc, char **argv)
 	/* Set the full flag to print the full path name if recursion is
 	** set and we are only displaying files or deleted files
 	*/
-	if ((flags & FS_FLAG_RECURSE) && (
-	  ((flags & FS_FLAG_UNALLOC) && (!(flags & FS_FLAG_ALLOC))) ||
+	if ((flags & FS_FLAG_NAME_RECURSE) && (
+	  ((flags & FS_FLAG_NAME_UNALLOC) && (!(flags & FS_FLAG_NAME_ALLOC))) ||
 	  ((localFlags & LCL_FILE) && (!(localFlags & LCL_DIR))) )) {
 
 		localFlags |= LCL_FULL;
