@@ -44,7 +44,7 @@ fs_data_alloc(u_int8_t type)
 	data->next = NULL;
 
 	if (type == FS_DATA_NONRES) {
-    		data->buflen = 0;
+		data->buflen = 0;
 		data->buf = NULL;
 		data->flags = (FS_DATA_NONRES | FS_DATA_INUSE);
 	}
@@ -307,7 +307,7 @@ fs_data_put_str(FS_DATA *data_head, char *name, u_int32_t type, u_int16_t id,
 FS_DATA *
 fs_data_put_run(FS_DATA *data_head, 
     u_int64_t start_vcn, u_int64_t runlen, FS_DATA_RUN *run,
-    char *name, u_int32_t type, u_int16_t id, u_int64_t size)
+    char *name, u_int32_t type, u_int16_t id, u_int64_t size, u_int8_t flags)
 {
 	FS_DATA *data;
 	FS_DATA_RUN *data_run, *data_run_prev;
@@ -326,7 +326,7 @@ fs_data_put_run(FS_DATA *data_head,
 		if (!data_head)
 			data_head = data;
 
-		data->flags = (FS_DATA_INUSE | FS_DATA_NONRES);
+		data->flags = (FS_DATA_INUSE | FS_DATA_NONRES | flags);
 		data->type = type;
 		data->id = id;
 		data->size = size;
@@ -336,6 +336,12 @@ fs_data_put_run(FS_DATA *data_head,
 
 		/* if this is not in the begining, then we need to make a filler 
 		 * to account for the cluster numbers we haven't seen yet
+		 *
+		 * This commonly happens when we process an MFT entry tha
+		 * is not a base entry and it is referenced in an $ATTR_LIST
+		 *
+		 * The $DATA attribute in the non-base have a non-zero
+		 * start_vcn.  
 		 */
 		if (start_vcn != 0) {
 			FS_DATA_RUN *fill = fs_data_run_alloc();
