@@ -10,7 +10,7 @@ extern FS_DATA *fs_data_put_str(FS_DATA *, char *, u_int32_t, u_int16_t,
   DADDR_T *, int);
 
 extern FS_DATA *fs_data_put_run(FS_DATA *, u_int64_t, u_int64_t,
-  FS_DATA_RUN *, char *, u_int32_t, u_int16_t, u_int64_t);
+  FS_DATA_RUN *, char *, u_int32_t, u_int16_t, u_int64_t, u_int8_t);
 
 extern FS_DATA *fs_data_lookup(FS_DATA *, u_int32_t, u_int16_t);
 
