@@ -26,174 +26,17 @@
 #include <sys/time.h>
 #include <time.h>
 
+#include "fs_os.h"
 #include "fs_types.h"
 
+
  /*
   * Verbose logging.
   */
 extern FILE *logfp;
 
- /*
-  * Solaris 2.x. Build for large files when dealing with filesystems > 2GB.
-  * With the 32-bit file model, needs pread() to access filesystems > 2GB.
-  */
-#if defined(SUNOS5)
-#define SUPPORTED
-#include <sys/sysmacros.h>
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define ROOTINO		UFSROOTINO
-#define STRTOUL		strtoul
-#define DADDR_T		daddr_t
-#define INO_TO_CG	itog
-#define DEF_FSTYPE	"solaris"
-
-#define u_int8_t	uint8_t
-#define u_int16_t	uint16_t
-#define u_int32_t	uint32_t
-#define u_int64_t	uint64_t
-
-#endif
-
- /*
-  * SunOS 4.x cannot handle filesystems > 2GB.
-  */
-#if defined(SUNOS4)
-#define SUPPORTED
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define STRTOUL		strtol
-#define DADDR_T		daddr_t
-#define DEF_FSTYPE	"solaris"
-#define INO_TO_CG	itog
-
-extern char *optarg;
-extern int optind;
-
-#define u_int8_t	uint8_t
-#define u_int16_t	uint16_t
-#define u_int32_t	uint32_t
-#define u_int64_t	uint64_t
-
-#endif
-
- /*
-  * FreeBSD can handle filesystems > 2GB.
-  */
-#if defined(FREEBSD2) || defined(FREEBSD3) || defined(FREEBSD4) || defined(FREEBSD5)
-#define SUPPORTED
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define STRTOUL		strtoul
-#define DADDR_T		daddr_t
-#define DEF_FSTYPE	"freebsd"
-#define INO_TO_CG	ino_to_cg
-#endif /* FREEBSD */
-
- /*
-  * BSD/OS can handle filesystems > 2GB.
-  */
-#if defined(BSDI2) || defined(BSDI3) || defined(BSDI4)
-#define SUPPORTED
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define STRTOUL		strtoul
-#define DADDR_T		daddr_t
-#define DEF_FSTYPE	"bsdi"
-#define INO_TO_CG	ino_to_cg
-#endif /* BSDI */
-
-/*
- * NetBSD
- */
-#if defined(NETBSD16)
-#define SUPPORTED
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define STRTOUL		strtoul
-#define DADDR_T		daddr_t
-#define DEF_FSTYPE	"netbsd"
-#define INO_TO_CG	ino_to_cg
-#endif /* NETBSD */
 
 
- /*
-  * OpenBSD looks like BSD/OS 3.x.
-  */
-#if defined(OPENBSD2) || defined (OPENBSD3)
-#define SUPPORTED
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define STRTOUL		strtoul
-#define DADDR_T		daddr_t
-#define DEF_FSTYPE	"openbsd"
-#define INO_TO_CG	ino_to_cg
-#endif
-
-#if defined(DARWIN)
-#define SUPPORTED
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define STRTOUL		strtoul
-#define DADDR_T		daddr_t
-#define DEF_FSTYPE	"darwin-hfs"
-#define INO_TO_CG	ino_to_cg
-#endif /* DARWIN */
-
-
- /*
-  * Linux 2.whatever. We'll see how stable the interfaces are.
-  */
-#if defined(LINUX2) 
-#define SUPPORTED
-#include <linux/types.h>
-#define USE_MYLSEEK
-#define HAVE_LLSEEK
-#define LSEEK		mylseek
-#define OFF_T		off_t
-#define STRTOUL		strtoul
-#define DADDR_T		daddr_t
-#define DEF_FSTYPE		"linux-ext2"
-#endif /* LINUX */
-
-#if defined(CYGWIN)
-#define SUPPORTED
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define STRTOUL		strtoul
-#define DADDR_T		daddr_t
-#define DEF_FSTYPE	"freebsd"
-#define INO_TO_CG	ino_to_cg
-
-#define roundup(x, y)	\
-	( ( ((x)+((y) - 1)) / (y)) * (y) )
-
-#endif /* CYGWIN */
-
-#if defined(__INTERNIX)
-#define SUPPORTED
-#define LSEEK		lseek
-#define OFF_T		off_t
-#define STRTOUL		strtoul
-#define DADDR_T		long 
-#define DEF_FSTYPE	"freebsd"
-#define INO_TO_CG	ino_to_cg
-
-#define roundup(x, y)	\
-	( ( ((x)+((y) - 1)) / (y)) * (y) )
-
-#endif /* INTERNIX*/
-
-
-
-
-
- /*
-  * Catch-all.
-  */
-#ifndef SUPPORTED
-#error "This operating system is not supported"
-#endif
 
 #ifndef NBBY				/* NIH */
 #define NBBY 8
@@ -377,9 +220,12 @@ struct FS_DATA {
 	u_int8_t	*buf;			/* buffer for resident data */
 };
 
-#define FS_DATA_INUSE	0x1
-#define FS_DATA_NONRES		0x2
-#define FS_DATA_RES		0x4
+#define FS_DATA_INUSE	0x1		// structre in use
+#define FS_DATA_NONRES	0x2		// non-resident 
+#define FS_DATA_RES		0x4		// resident 
+#define	FS_DATA_ENC		0x10	// encrypted 
+#define FS_DATA_COMP	0x20	// compressed
+#define FS_DATA_SPAR	0x40	// sparse
 
 
 /* Currently this is only used with NTFS & FAT systems */
@@ -391,20 +237,20 @@ struct FS_NAME {
 };
 
 struct FS_INODE {
-    mode_t  mode;			/* type and permission */
-    int     nlink;			/* link count */
-    OFF_T   size;			/* file size */
-    uid_t   uid;			/* owner */
-    gid_t   gid;			/* group */
-    time_t  mtime;			/* last modified */
-    time_t  atime;			/* last access */
-    time_t  ctime;			/* last status change */
+	mode_t  mode;			/* type and permission */
+	int     nlink;			/* link count */
+	OFF_T   size;			/* file size */
+	uid_t   uid;			/* owner */
+	gid_t   gid;			/* group */
+	time_t  mtime;			/* last modified */
+	time_t  atime;			/* last access */
+	time_t  ctime;			/* last status change */
 	time_t	crtime;		/* create time (NTFS only: not used by ils) */
-    time_t  dtime;			/* delete time (Linux only) */
-    DADDR_T *direct_addr;		/* direct blocks */
-    int     direct_count;		/* number of blocks */
-    DADDR_T *indir_addr;		/* indirect blocks */
-    int     indir_count;		/* number of blocks */
+	time_t  dtime;			/* delete time (Linux only) */
+	DADDR_T *direct_addr;		/* direct blocks */
+	int     direct_count;		/* number of blocks */
+	DADDR_T *indir_addr;		/* indirect blocks */
+	int     indir_count;		/* number of blocks */
 
 	u_int32_t	seq;			/* sequence number (NTFS Only) */
 	FS_DATA	*attr;				/* additional attributes for NTFS */
@@ -419,20 +265,22 @@ extern FS_INODE *fs_inode_realloc(FS_INODE *, int, int);
 extern void fs_inode_free(FS_INODE *);
 
 
+
+
 /************************************************************************* 
  * Directory entries 
  */
 struct FS_DENT {
-    unsigned int inode;     /* inode number */
-    u_int16_t nextLink;  	/* distance until next entry
-							** according to the actual entry ptr */
-    u_int16_t reclen;    	/* actual length of record based on name len */
-    u_int8_t ent_type; 		/* dir, file etc FS_DENT_???*/
-    u_int16_t namlen;   	/* length of actual name */
-    ULONG maxnamlen;		/* size allocated to name*/
-    char *name;     		/* name of file, directory etc
+	unsigned int inode;     /* inode number */
+	u_int16_t nextLink;  	/* distance until next entry
+				** according to the actual entry ptr */
+	u_int16_t reclen;    	/* actual length of record based on name len */
+	u_int8_t ent_type; 		/* dir, file etc FS_DENT_???*/
+	u_int16_t namlen;   	/* length of actual name */
+	ULONG maxnamlen;		/* size allocated to name*/
+	char *name;     		/* name of file, directory etc
                 			** this MUST be NULL terminated */
-    struct FS_INODE *fsi;   /* Inode structure */
+	struct FS_INODE *fsi;   /* Inode structure */
 	char *path;				/* prefix to name when recursing*/
 	unsigned int pathdepth;	/* current depth of directories*/
 };
@@ -578,7 +426,7 @@ extern u_int8_t	guessu32(FS_INFO *, u_int8_t *, u_int32_t);
 
 /* 16-bit values */
 #define getu16(fs, x)   \
-    ((fs->flags & FS_LIT_ENDIAN) ? \
+    (u_int16_t)((fs->flags & FS_LIT_ENDIAN) ? \
 	  (((u_int8_t *)x)[0] + (((u_int8_t *)x)[1] << 8)) :    \
 	  (((u_int8_t *)x)[1] + (((u_int8_t *)x)[0] << 8)) ) 
 
@@ -587,7 +435,7 @@ extern u_int8_t	guessu32(FS_INFO *, u_int8_t *, u_int32_t);
 
 /* 32-bit values */
 #define getu32(fs, x)	\
-	( (fs->flags & FS_LIT_ENDIAN)  ?	\
+	(u_int32_t)( (fs->flags & FS_LIT_ENDIAN)  ?	\
      ((((u_int8_t *)x)[0] <<  0) + \
 	  (((u_int8_t *)x)[1] <<  8) + \
 	  (((u_int8_t *)x)[2] << 16) + \
@@ -602,7 +450,7 @@ extern u_int8_t	guessu32(FS_INFO *, u_int8_t *, u_int32_t);
 	((int32_t)getu32(fs, x))
 
 #define getu48(fs, x)   \
-	( (fs->flags & FS_LIT_ENDIAN)  ?	\
+	(u_int64_t)( (fs->flags & FS_LIT_ENDIAN)  ?	\
       ((u_int64_t) \
 	  ((u_int64_t)((u_int8_t *)(x))[0] <<  0)+ \
 	  ((u_int64_t)((u_int8_t *)(x))[1] <<  8) + \
@@ -621,7 +469,7 @@ extern u_int8_t	guessu32(FS_INFO *, u_int8_t *, u_int32_t);
 
 
 #define getu64(fs, x)   \
-	( (fs->flags & FS_LIT_ENDIAN)  ?	\
+	(u_int64_t)( (fs->flags & FS_LIT_ENDIAN)  ?	\
       ((u_int64_t) \
 	  ((u_int64_t)((u_int8_t *)(x))[0] << 0)  + \
 	  ((u_int64_t)((u_int8_t *)(x))[1] << 8) + \
