@@ -5,7 +5,7 @@
 ** Identify the type of file system being used
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -34,7 +34,9 @@ FS_TYPES fs_open_table[] = {
     {"netbsd", FFS_1, "NetBSD FFS"},
 	{"ntfs", NTFS, "NTFS"},
     {"openbsd", FFS_1, "OpenBSD FFS"},
+	{"raw", RAW, "Raw Data"},
 	{"solaris", FFS_2, "Solaris FFS"},
+	{"swap", SWAP, "Swap Space"},
 	{0},
 };
 
