@@ -52,6 +52,7 @@
 #include "mymalloc.h"
 
 static DADDR_T block = 0;			/* the block to find */
+static INUM_T parinode = 0;
 static INUM_T curinode;			/* the inode being analyzed */
 
 static u_int32_t curtype;		/* the type currently being analyzed: NTFS */
@@ -64,18 +65,24 @@ static char *path;
 
 FILE *logfp;
 
-#define FIND_ALL	0x1
-#define FOUND_ONE	0x2
-#define USE_PATH	0x4
+#define FIND_ALL	0x01
+#define FOUND_ONE	0x02
+#define USE_PATH	0x04
+#define USE_DATA	0x08
+#define USE_PAR		0x10
+#define PAR_LONG	0x20
 static u_int8_t localflags;
 
 static void usage(char *prog) {
-	printf("usage: %s [-avV] [-f fstype] [-d unit_addr] [-n file] image\n", prog);
+	printf("usage: %s [-alvV] [-f fstype] [-d unit_addr] [-n file] [-p par_addr] [-z ZONE] image\n", prog);
 	printf("\t-a: find all inodes\n");
 	printf("\t-d unit_addr: Find the meta data given the data unit\n");
+	printf("\t-l: long format when -p is given\n");
 	printf("\t-n file: Find the meta data given the file name\n");
+	printf("\t-p par_addr: Find UNALLOCATED MFT entries given the parent's meta address (NTFS only)\n");
 	printf("\t-v: Verbose output to stderr\n");
 	printf("\t-V: Print version\n");
+	printf("\t-z ZONE: Time zone setting when -l -p is given\n");
 	printf("\t-f fstype: Image file system type\n");
 	printf("Supported file system types:\n");
     fs_print_types();
@@ -85,6 +92,41 @@ static void usage(char *prog) {
 
 
 
+FS_DENT *fs_dent;
+
+/* dent call back for finding unallocated files based on parent directory
+ */
+static u_int8_t
+find_inode_parent(FS_INFO *fs, INUM_T inum, FS_INODE *fsi, int flags, char *ptr) 
+{
+	FS_NAME *fs_name;
+
+	/* go through each file name structure */
+	fs_name = fsi->name;
+	while (fs_name) {
+		if (fs_name->par_inode == parinode) {
+
+			fs_dent->fsi = fsi;
+			fs_dent->inode = inum;
+			strncpy(fs_dent->name, fs_name->name, fs_dent->maxnamlen);
+			if (localflags & PAR_LONG) {
+				fs_dent_print_long(stdout, fs_dent, FS_FLAG_NAME_UNALLOC, fs, NULL);
+			}
+			else  {
+				fs_dent_print(stdout, fs_dent, FS_FLAG_NAME_UNALLOC, fs, NULL);
+				printf("\n");
+			}
+			fs_dent->fsi = NULL;
+			localflags |= FOUND_ONE;
+		}
+		fs_name = fs_name->next;
+	}
+
+	return WALK_CONT;
+}
+
+
+
 
 /* 
  * dent_walk for finding the inode based on path
@@ -392,22 +434,42 @@ main(int argc, char **argv)
 
 	localflags = 0;
 
-    while ((ch = getopt(argc, argv, "ad:f:n:vV")) > 0) {  
+	while ((ch = getopt(argc, argv, "ad:f:ln:p:vVz:")) > 0) {  
 		switch (ch) {
 		 case 'a':
 			localflags |= FIND_ALL;
 			break;
 		case 'd':
+			if (localflags & (USE_PAR | USE_PATH)) {
+				fprintf(stderr, "error: only one address type can be given\n");
+				usage(argv[0]);
+			}
+			localflags |= USE_DATA;
 			block = atoi(optarg);
 			break;
-        case 'f':
-            fstype = optarg;
-            break;
-        case 'n':
+	    	case 'f':
+			fstype = optarg;
+			break;
+	    	case 'l':
+			localflags |= PAR_LONG;
+			break;
+		case 'n':
+			if (localflags & (USE_PAR | USE_DATA)) {
+				fprintf(stderr, "error: only one address type can be given\n");
+				usage(argv[0]);
+			}
 			localflags |= USE_PATH;
 			path = mymalloc (strlen (optarg) + 1);
 			strncpy (path, optarg, strlen(optarg) + 1);
-            break;
+			break;
+		case 'p':
+			if (localflags & (USE_PATH | USE_DATA)) {
+				fprintf(stderr, "error: only one address type can be given\n");
+				usage(argv[0]);
+			}
+			localflags |= USE_PAR;
+			parinode = atoi(optarg);
+			break;
 		case 'v':
 			verbose++;
 			logfp = stderr;
@@ -415,14 +477,26 @@ main(int argc, char **argv)
 		case 'V':
 			print_version();
 			exit(0);
+		case 'z':
+		{
+			char envstr[32];
+			snprintf(envstr, 32, "TZ=%s", optarg);
+			if (0 != putenv(envstr)) {
+				error ("error setting environment");
+			}
+
+			/* we should be checking this somehow */
+			tzset();
+			break;
+		}
 		case '?':  
 			default:
 			usage(argv[0]);
 		}
 	}
 	
-	if (optind ==  argc) {
-		fprintf(stderr, "Missing image name\n");
+	if ((optind + 1) !=  argc) {
+		fprintf(stderr, "Missing image name or too many arguments\n");
 		usage(argv[0]);
 	}
 
@@ -430,11 +504,6 @@ main(int argc, char **argv)
 
 	if (localflags & USE_PATH) {
 
-		if (block != 0) {
-			fprintf (stderr, "-d and -n must not be given together\n");
-			exit(1);
-		}
-
 		if (localflags & FIND_ALL) {
 			fprintf (stderr, "-a and -n must not be given together\n");
 			exit(1);
@@ -469,24 +538,8 @@ main(int argc, char **argv)
 		}
 
 	}
-	else {
+	else if (localflags & USE_DATA) {
 
-		/* The old syntax was given */
-		if (block == 0) {
-
-			if (optind + 1 != argc) 
-				usage(argv[0]);
-
-			fprintf (stderr, 
-			  "NOTE: This syntax should no longer be used.  Use the '-d' argument instead.\n");
-
-			block = atoi(argv[optind]);
-		}
-		else {
-			if (optind < argc) {
-				usage(argv[0]);
-			}
-		}
 
 		if (block > fs->last_block) {
 			fprintf(stderr, 
@@ -518,6 +571,30 @@ main(int argc, char **argv)
 				block_act, (char *) 0);
 		}
 	}
+	else if (localflags & USE_PAR) {
+		if ((fs->ftype & FSMASK) != NTFS_TYPE) {
+			fprintf(stderr, "-p works only with NTFS file systems\n");
+			fs->close(fs);
+			exit(1);
+		}
+
+		else if (parinode > fs->last_inum) {
+			fprintf(stderr, 
+			  "Meta data %lu is larger than last MFT entry in image (%lu)\n",
+			  (ULONG)parinode, (ULONG)fs->last_inum);
+			fs->close(fs);
+			exit (1);
+		}
+
+		fs_dent = fs_dent_alloc(256);
+
+		fs->inode_walk(fs, fs->first_inum, fs->last_inum,
+	  	  FS_FLAG_META_LINK | FS_FLAG_META_UNLINK | 
+	  	  FS_FLAG_META_UNALLOC | FS_FLAG_META_USED, 
+		  find_inode_parent, (char *) 0);
+
+		fs_dent_free(fs_dent);
+	}
 
 	fs->close(fs);
 
