@@ -25,6 +25,11 @@
 #include <errno.h>
 #include <syscall.h>
 
+/* Needed for some Linux distros - Fedora Core 2 */
+#if defined(LINUX2)
+#include <linux/unistd.h>
+#endif
+
  /*
   * This is LINUX, live on the bleeding edge and watch your software break
   * with the next release...
