@@ -90,7 +90,7 @@ u_int32_t
 nt2unixtime(u_int64_t ntdate)
 {
 // (369*365 + 89) * 24 * 3600 * 10000000
-#define	NSEC_BTWN_1601_1970	(u_int64_t)(116444736000000000)
+#define	NSEC_BTWN_1601_1970	(u_int64_t)(116444736000000000ULL)
 
 	ntdate -= (u_int64_t)NSEC_BTWN_1601_1970;
 	ntdate /= (u_int64_t)10000000;
@@ -403,12 +403,13 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
 	if (runlen)
 		*runlen = 0;
 
-	/* the ntfs runlist has dynamic size integers, specified by the
-	 * len_size and off_size values
+	/* Cycle through each run in the runlist 
+	 * We got until we find an entry for a run with no length
+	 * An entry with offset of 0 is for sparse runs
 	 */
-	while ((NTFS_RUNL_LENSZ(run) != 0) && (NTFS_RUNL_OFFSZ(run) != 0) ) {
+	while (NTFS_RUNL_LENSZ(run) != 0) {
 
-		/* allocate a new data_run */
+		/* allocate a new fs_data_run */
 		data_run = fs_data_run_alloc();
 
 		/* make the list, unless its the first pass & then we set the head */
@@ -464,10 +465,18 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
 		/* The NT 4.0 version of NTFS uses an offset of -1 to represent
 		 * a hole, so add the sparse flag and make it look like the 2K
 		 * version with a offset of 0
+		 *
+		 * A user reported an issue where the $Bad file started with
+		 * its offset as -1 and it was not NT (maybe a conversion)
+		 * Change the check now to not limit to NT, but make sure
+		 * that it is the first run
 		 */
-		if ((data_run->addr == -1) && (ntfs->ver == NTFS_VINFO_NT)) {
+		if (((data_run->addr == -1) && (prev_addr == 0)) ||  
+		  ((data_run->addr == -1) && (ntfs->ver == NTFS_VINFO_NT))) {
 			data_run->flags |= FS_DATA_SPARSE;
 			data_run->addr = 0;
+			if (verbose)
+				fprintf(logfp, "ntfs_make_data_run: Sparse Run\n");
 		}
 
 		/* A Sparse file has a run with an offset of 0
@@ -497,10 +506,11 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
 
 	}
 
-	/* special case, only seen on the $BADCLUST mft entry, which 
-	 * always has a weird size.
-	 * Basically, we check if there is only one run entry and it
-	 * is sparse, then get rid of it
+	/* special case for $BADCLUST, which is a sparse file whose size is
+	 * the entire file system.
+	 *
+	 * If there is only one run entry and it is sparse, then there are no
+	 * bad blocks, so get rid of it.
 	 */
 	if ((data_run_head != NULL) && (data_run_head->next == NULL) && 
 	  (data_run_head->flags & FS_DATA_SPARSE)) {
@@ -528,7 +538,7 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
  *
  * No notion of META
  */
-static void
+void
 ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data, 
 	int flags, FS_FILE_WALK_FN action, char *ptr)
 {
@@ -560,6 +570,13 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 		u_int64_t fsize;
 		FS_DATA_RUN *fs_data_run;
 
+
+		if ( ((flags & FS_FLAG_FILE_NOABORT) == 0) && 
+		  (fs_data->flags & FS_DATA_COMP) ) {
+			error ("ERROR: TSK Cannot process compressed files");
+		}
+
+
 		/* if we want the slack space too, then use the runlen  */
 		if (flags & FS_FLAG_FILE_SLACK)
 			fsize = fs_data->runlen;
@@ -576,12 +593,18 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 		/* cycle through the number of runs we have */
 		while (fs_data_run) {
 
-			/* if this is just a filler entry, then skip it
-			 * This should not happen, but may when dealing with deleted 
-			 * entries 
+			/* We may get a FILLER entry at the beginning of the run
+			 * if we are processing a non-base file record because
+			 * this $DATA attribute could not be the first in the bigger
+			 * attribute. Therefore, do not error if it starts at 0
 			 */
-			if (fs_data_run->flags & FS_DATA_FILLER) 
-				error("Filler Entry exists in fs_data_run");
+			if (fs_data_run->flags & FS_DATA_FILLER) {
+				if (fs_data_run->addr != 0)
+					error("Filler Entry exists in fs_data_run %lu for %lu",
+					  (ULONG)fs_data_run->addr, (ULONG)fs_data_run->len);
+				else
+					fs_data_run = fs_data_run->next;
+			}
 
 			addr = fs_data_run->addr;	
 
@@ -690,7 +713,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 	FS_INFO 	*fs = (FS_INFO *)&ntfs->fs_info;
 
 	if (verbose)
-		fprintf(logfp, "ntfs_proc_attrseq: Processing MFT %lu\n", 
+		fprintf(logfp, "ntfs_proc_attrseq: Processing MFT %lu (maybe)\n", 
 		  (ULONG)ntfs->mnum);
 
 	/* Cycle through the list of attributes */
@@ -713,7 +736,10 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 
 			/* Copy the name into ASCII */
 			if (attr->nlen)
-				uni2ascii((char *)&attr->c.r.name, attr->nlen, name, sizeof(name));
+				uni2ascii(
+				  (char *)(int)attr + getu16(fs, attr->name_off), 
+				  attr->nlen, name, sizeof(name));
+
 			/* Call the unnamed $Data attribute, $Data */
 			else if (type == NTFS_ATYPE_DATA)
 				strncpy(name, "$Data", NTFS_MAXNAMLEN+1);
@@ -739,10 +765,14 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 		 */
 		else {
 			FS_DATA_RUN *fs_data_run;
+			u_int8_t	data_flag = 0;
 
 			/* Copy the name */
 			if (attr->nlen)
-				uni2ascii((char *)&attr->c.nr.name, attr->nlen, name, sizeof(name));
+				uni2ascii(
+				  (char *)(int)attr + getu16(fs, attr->name_off), 
+				  attr->nlen, name, sizeof(name));
+
 			else if (type == NTFS_ATYPE_DATA)
 				strncpy(name, "$Data", NTFS_MAXNAMLEN+1);
 			else
@@ -758,11 +788,21 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 			  (ntfs_runlist *) ((int)attr + getu16(fs, attr->c.nr.run_off)),
 			  &runlen);
 
+
+			/* Determine the flags based on compression and stuff */
+			data_flag = 0;
+			if (getu16(fs, attr->flags) & NTFS_ATTR_FLAG_COMP)
+				data_flag |= FS_DATA_COMP;
+			if (getu16(fs, attr->flags) & NTFS_ATTR_FLAG_ENC)
+				data_flag |= FS_DATA_ENC;
+			if (getu16(fs, attr->flags) & NTFS_ATTR_FLAG_SPAR)
+				data_flag |= FS_DATA_SPAR;
+
 			/* Add the run to the list */
 			fs_inode->attr = fs_data_put_run(fs_inode->attr, 
 			  getu64(fs, attr->c.nr.start_vcn), runlen, fs_data_run,
 			  name, type, getu16(fs, attr->id), 
-			  getu64(fs, attr->c.nr.ssize));
+			  getu64(fs, attr->c.nr.ssize), data_flag);
 		}
 
 
@@ -867,7 +907,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 		/* we account for the size of $Data, and directory locations */
 		if ( (fs_data->type == NTFS_ATYPE_DATA) ||
 		  (fs_data->type == NTFS_ATYPE_IDXROOT) ||
-		  (fs_data->type == NTFS_ATYPE_IDXALLOC) )
+		  (fs_data->type == NTFS_ATYPE_IDXALLOC) ) 
 			fs_inode->size += fs_data->size;
 	}
 
@@ -917,32 +957,6 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 }
 
 
-/*
- * Print the UNIX mode value in the DOS mode format
- */
-static void 
-print_dosmode (FILE *hFile, int mode)
-{
-	fprintf(hFile, "DOS Mode: ");
-
-    if (mode & FS_INODE_DIR)
-        fprintf(hFile, "Directory");  
-    else
-        fprintf(hFile, "File");
-
-    
-    if ((mode & (MODE_IRUSR | MODE_IRGRP | MODE_IROTH)) == 0)
-        fprintf(hFile, ", Read Only");
-    
-    if ((mode & (MODE_IWUSR | MODE_IWGRP | MODE_IWOTH)) == 0)
-        fprintf(hFile, ", Hidden");
- 
-    fprintf(hFile, "\n");
-
-    return;
-}
-
-
 
 /********   Attribute List Action and Function ***********/
 
@@ -1070,7 +1084,7 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 		ntfs_mft_lookup(ntfs, mft, mftnum);
 
 		/* verify that this entry refers to the original one */
-		if (getu48(fs, mft->file_ref) != ntfs->mnum) {
+		if (getu48(fs, mft->base_ref) != ntfs->mnum) {
 
 			/* Before we raise alarms, check if the original was
 			 * unallocated.  If so, then the list entry could 
@@ -1081,7 +1095,7 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 			else
 				error ("Extension record %lu (file ref = %llu) is not for attribute list of %lu", 
 			  (ULONG)mftnum, 
-			  (ULLONG)getu48(fs, mft->file_ref),
+			  (ULLONG)getu48(fs, mft->base_ref),
 			  (ULONG)ntfs->mnum);
 
 		}	
@@ -1517,7 +1531,7 @@ ntfs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 		/* we only want to look at base file records 
 		 * (extended are because the base could not fit into one)
 		 */
-		if (getu48(fs, ntfs->mft->file_ref) != NTFS_MFT_BASE)
+		if (getu48(fs, ntfs->mft->base_ref) != NTFS_MFT_BASE)
 			continue;
 
 		/* NOTE: We could add a sanity check here with the MFT bitmap
@@ -1576,9 +1590,15 @@ ntfs_fsstat(FS_INFO *fs, FILE *hFile)
 
 	fprintf(hFile, "File System Type: NTFS\n");
 
-	fprintf(hFile, "Volume Serial Number: %X%X\n", 
-	  (u_int32_t)(getu64(fs, ntfs->fs->serial) & 0xffffffff00000000 >> 32),
-	  (u_int32_t)getu64(fs, ntfs->fs->serial) & 0xffffffff);
+	fprintf(hFile, "Volume Serial Number: %.16"PRIX64"\n", 
+	  getu64(fs, ntfs->fs->serial));
+
+
+	fprintf(hFile, "OEM Name: %c%c%c%c%c%c%c%c\n", 
+	  ntfs->fs->oemname[0],
+	  ntfs->fs->oemname[1], ntfs->fs->oemname[2], ntfs->fs->oemname[3], 
+	  ntfs->fs->oemname[4],
+	  ntfs->fs->oemname[5], ntfs->fs->oemname[6], ntfs->fs->oemname[7]);
 
 
 	/*
@@ -1610,11 +1630,23 @@ ntfs_fsstat(FS_INFO *fs, FILE *hFile)
     fprintf(hFile, "\nMETA-DATA INFORMATION\n");
     fprintf(hFile, "--------------------------------------------\n");
 
+	fprintf(hFile, "First Cluster of MFT: %"PRIu64"\n",
+	  getu64(fs, ntfs->fs->mft_clust));
+
+	fprintf(hFile, "First Cluster of MFT Mirror: %"PRIu64"\n",
+	  getu64(fs, ntfs->fs->mftm_clust));
+
+	fprintf(hFile, "Size of MFT Entries: %"PRIu32" bytes\n",
+		ntfs->mft_rsize_b);
+	fprintf(hFile, "Size of Index Records: %"PRIu32" bytes\n",
+		ntfs->idx_rsize_b);
+
     fprintf(hFile, "Range: %lu - %lu\n", (ULONG)fs->first_inum,
       (ULONG)fs->last_inum);
     fprintf(hFile, "Root Directory: %lu\n", (ULONG)fs->root_inum);
 
 
+
     fprintf(hFile, "\nCONTENT-DATA INFORMATION\n");
     fprintf(hFile, "--------------------------------------------\n");
     fprintf(hFile, "Sector Size: %d\n", ntfs->ssize_b);
@@ -1624,11 +1656,13 @@ ntfs_fsstat(FS_INFO *fs, FILE *hFile)
 	  (ULONG)fs->first_block,
       (ULONG)fs->last_block);
 
+	fprintf(hFile, "Total Sector Range: 0 - %"PRIu64"\n",
+	  getu64(fs, ntfs->fs->vol_size_s) - 1);
 	
 	/* 
 	 * Attrdef Info 
 	 */
-	fprintf(hFile, "\nAttribute Defs (MFT Entry: %d)\n", NTFS_MFT_ATTR);
+	fprintf(hFile, "\n$AttrDef Attribute Values:\n");
 
 	if (!ntfs->attrdef)
 		ntfs_load_attrdef(ntfs);
@@ -1637,11 +1671,33 @@ ntfs_fsstat(FS_INFO *fs, FILE *hFile)
 
 	while (getu32(fs, attrdeftmp->type)) {
 		uni2ascii((char *)attrdeftmp->label, 128, asc, 128);
-		printf("%s: %d\n", asc, getu32(fs, attrdeftmp->type));
+		fprintf(hFile, "%s (%"PRIu32")   ",
+		  asc, 
+		  getu32(fs, attrdeftmp->type));
+		
+	
+		if ((getu64(fs, attrdeftmp->minsize) == 0) && 
+		  (getu64(fs, attrdeftmp->maxsize) == 0xffffffffffffffffULL)) {
+
+			fprintf(hFile, "Size: No Limit");
+		} 
+		else {
+			fprintf(hFile, "Size: %"PRIu64"-%"PRIu64,
+			  getu64(fs, attrdeftmp->minsize),
+			  getu64(fs, attrdeftmp->maxsize));
+		}
+
+		fprintf(hFile, "   Flags: %s%s%s\n", 
+		  (getu32(fs, attrdeftmp->flags)&NTFS_ATTRDEF_FLAGS_RES?"Resident":""),
+		  (getu32(fs, attrdeftmp->flags)&NTFS_ATTRDEF_FLAGS_NONRES?"Non-resident":""), 
+		  (getu32(fs, attrdeftmp->flags)&NTFS_ATTRDEF_FLAGS_IDX?",Index":"")); 
 
 		attrdeftmp++;
 	}
 
+
+
+
 }
 
 
@@ -1674,68 +1730,227 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 {
     FS_INODE *fs_inode;
 	FS_DATA *fs_data;
+	NTFS_INFO *ntfs = (NTFS_INFO *)fs;
 
     fs_inode = ntfs_inode_lookup (fs, inum);
-    fprintf(hFile, "MFT Entry: %lu\n", (ULONG) inum);
-	fprintf(hFile, "Sequence: %lu\n", (ULONG) fs_inode->seq);
-    fprintf(hFile, "%sAllocated\n",
-      (fs_inode->flags & FS_FLAG_META_ALLOC)? "" : "Not ");
+	fprintf(hFile, "MFT Entry Header Values:\n");
+    fprintf(hFile, "Entry: %lu        Sequence: %lu\n", 
+	   (ULONG) inum, (ULONG)fs_inode->seq);
 
-	fprintf(hFile, "UID: %d\n", (int)fs_inode->uid);
+	if (getu48(fs, ntfs->mft->base_ref) != 0) {
+		fprintf(hFile, "Base File Record: %"PRIu64"\n",
+		  (u_int64_t)getu48(fs, ntfs->mft->base_ref));
+	}
+
+	fprintf(hFile, "$LogFile Sequence Number: %"PRIu64"\n",
+	  getu64(fs, ntfs->mft->lsn));
+
+    fprintf(hFile, "%sAllocated %s\n",
+      (fs_inode->flags & FS_FLAG_META_ALLOC)? "" : "Not ",
+      (fs_inode->mode & FS_INODE_DIR)? "Directory" : "File");
 
-    print_dosmode(hFile, fs_inode->mode);
-   
-    fprintf(hFile, "Size: %llu\n", (ULLONG) fs_inode->size);
     fprintf(hFile, "Links: %lu\n", (ULONG) fs_inode->nlink);
 
-    if (fs_inode->name) {
-        FS_NAME *fs_name = fs_inode->name;
-        fprintf(hFile, "Name: ");
-        while (fs_name) {
-            fprintf(hFile, "%s", fs_name->name);
-            fs_name = fs_name->next;
-            if (fs_name)
-                fprintf(hFile, ", ");
-            else
-                fprintf(hFile, "\n");
-        }
-    }
 
-	/* Standard Info Times */
-    if (sec_skew != 0) {
-		fprintf(hFile, "\nAdjusted $STANDARD_INFORMATION Times:\n");
-        fs_inode->mtime -= sec_skew;
-        fs_inode->atime -= sec_skew;
-        fs_inode->ctime -= sec_skew;
-        fs_inode->crtime -= sec_skew;
-   
-    	fprintf(hFile, "Created:\t%s", ctime(&fs_inode->crtime));
-    	fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
-    	fprintf(hFile, "MFT Modified:\t%s", ctime(&fs_inode->ctime));
-    	fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
+	/* STANDARD_INFORMATION info */
+	fs_data = fs_data_lookup(fs_inode->attr, NTFS_ATYPE_SI, 0);
+	if (fs_data) {
+        ntfs_attr_si *si = (ntfs_attr_si *)fs_data->buf;
+		int a = 0;
 
-        fs_inode->mtime += sec_skew;
-        fs_inode->atime += sec_skew;
-        fs_inode->ctime += sec_skew;
-        fs_inode->crtime += sec_skew;
+		fprintf(hFile, "\n$STANDARD_INFORMATION Attribute Values:\n");
 
-		fprintf(hFile, "\nOriginal $STANDARD_INFORMATION Times:\n");
-    }
-    else
-		fprintf(hFile, "\n$STANDARD_INFORMATION Times:\n");
+		fprintf(hFile, "Flags: ");
+		if (getu32(fs, si->dos) & NTFS_SI_RO)
+			fprintf(hFile, "%sRead Only",
+			  a++ == 0 ? "" : ", ");
 
-    fprintf(hFile, "Created:\t%s", ctime(&fs_inode->crtime));
-    fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
-    fprintf(hFile, "MFT Modified:\t%s", ctime(&fs_inode->ctime));
-    fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
+		if (getu32(fs, si->dos) & NTFS_SI_HID)
+			fprintf(hFile, "%sHidden",
+			  a++ == 0 ? "" : ", ");
 
-	/* fname Times */
+		if (getu32(fs, si->dos) & NTFS_SI_SYS)
+			fprintf(hFile, "%sSystem",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_ARCH)
+			fprintf(hFile, "%sArchive",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_DEV)
+			fprintf(hFile, "%sDevice",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_NORM)
+			fprintf(hFile, "%sNormal",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_TEMP)
+			fprintf(hFile, "%sTemporary",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_SPAR)
+			fprintf(hFile, "%sSparse",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_REP)
+			fprintf(hFile, "%sReparse Point",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_COMP)
+			fprintf(hFile, "%sCompressed",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_OFF)
+			fprintf(hFile, "%sOffline",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_NOIDX)
+			fprintf(hFile, "%sNot Content Indexed",
+			  a++ == 0 ? "" : ", ");
+
+		if (getu32(fs, si->dos) & NTFS_SI_ENC)
+			fprintf(hFile, "%sEncrypted", 
+			  a++ == 0 ? "" : ", ");
+
+		fprintf(hFile, "\n");
+
+		fprintf(hFile, "Owner ID: %lu     Security ID: %lu\n",
+		  (ULONG)getu32(fs, si->own_id), 
+		  (ULONG)getu32(fs, si->sec_id));
+
+		if (getu32(fs, si->maxver) != 0) {
+			fprintf(hFile, "Version %d of %d\n",
+			  getu32(fs, si->ver), getu32(fs, si->maxver));
+		}
+
+		if (getu64(fs, si->quota) != 0) {
+			fprintf(hFile, "Quota Charged: %lu\n",
+			  (ULONG)getu64(fs, si->quota));
+		}
+
+		if (getu64(fs, si->usn) != 0) {
+			fprintf(hFile, "Last User Journal Update Sequence Number: %lu\n",
+			  (ULONG)getu64(fs, si->usn));
+		}
+
+
+		/* Times - take it from fs_inode instead of redoing the work */
+
+	    if (sec_skew != 0) {
+			fprintf(hFile, "\nAdjusted times:\n");
+			fs_inode->mtime -= sec_skew;
+			fs_inode->atime -= sec_skew;
+			fs_inode->ctime -= sec_skew;
+			fs_inode->crtime -= sec_skew;
+
+			fprintf(hFile, "Created:\t%s", ctime(&fs_inode->crtime));
+			fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
+			fprintf(hFile, "MFT Modified:\t%s", ctime(&fs_inode->ctime));
+			fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
+
+			fs_inode->mtime += sec_skew;
+			fs_inode->atime += sec_skew;
+			fs_inode->ctime += sec_skew;
+			fs_inode->crtime += sec_skew;
+
+			fprintf(hFile, "\nOriginal times:\n");
+		}
+
+		fprintf(hFile, "Created:\t%s", ctime(&fs_inode->crtime));
+		fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
+		fprintf(hFile, "MFT Modified:\t%s", ctime(&fs_inode->ctime));
+		fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
+	}
+
+	/* $FILE_NAME Information */
 	fs_data = fs_data_lookup(fs_inode->attr, NTFS_ATYPE_FNAME, 0);
 	if(fs_data) {
+
         ntfs_attr_fname *fname = (ntfs_attr_fname *)fs_data->buf;
 		time_t cr_time, m_time, c_time, a_time;
+		u_int64_t flags;
+		int a = 0;
+
+		fprintf(hFile, "\n$FILE_NAME Attribute Values:\n");
 
 
+		flags = getu64(fs, fname->flags);
+		fprintf(hFile, "Flags: ");
+
+		if (flags & NTFS_FNAME_FLAGS_DIR)
+            fprintf(hFile, "%sDirectory",
+			  a++ == 0 ? "" : ", ");
+		if (flags & NTFS_FNAME_FLAGS_DEV)
+            fprintf(hFile, "%sDevice",
+			  a++ == 0 ? "" : ", ");
+		if (flags & NTFS_FNAME_FLAGS_NORM)
+            fprintf(hFile, "%sNormal",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_RO)
+            fprintf(hFile, "%sRead Only",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_HID)
+            fprintf(hFile, "%sHidden",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_SYS)
+            fprintf(hFile, "%sSystem",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_ARCH)
+            fprintf(hFile, "%sArchive",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_TEMP)
+            fprintf(hFile, "%sTemp",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_SPAR)
+            fprintf(hFile, "%sSparse",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_REP)
+            fprintf(hFile, "%sReparse Point",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_COMP)
+            fprintf(hFile, "%sCompressed",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_ENC)
+            fprintf(hFile, "%sEncrypted",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_OFF)
+            fprintf(hFile, "%sOffline",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_NOIDX)
+            fprintf(hFile, "%sNot Content Indexed",
+			  a++ == 0 ? "" : ", ");
+        if (flags & NTFS_FNAME_FLAGS_IDXVIEW)
+            fprintf(hFile, "%sIndex View",
+			  a++ == 0 ? "" : ", ");
+
+        fprintf(hFile, "\n");
+
+		/* We could look this up in the attribute, but we already did
+		 * the work */
+	    if (fs_inode->name) {
+			FS_NAME *fs_name = fs_inode->name;
+			fprintf(hFile, "Name: ");
+			while (fs_name) {
+				fprintf(hFile, "%s", fs_name->name);
+				fs_name = fs_name->next;
+				if (fs_name)
+					fprintf(hFile, ", ");
+				else
+					fprintf(hFile, "\n");
+			}
+		}
+
+		fprintf(hFile, "Parent MFT Entry: %"PRIu64" \tSequence: %"PRIu16"\n",
+		  (u_int64_t)getu48(fs,fname->par_ref), getu16(fs,fname->par_seq));
+
+		fprintf(hFile, "Allocated Size: %"PRIu64"   \tActual Size: %"PRIu64"\n",
+		  getu64(fs,fname->alloc_fsize), getu64(fs,fname->real_fsize));
+
+
+		/* 
+		 * Times 
+		 */
 		cr_time = nt2unixtime(getu64(fs, fname->crtime));
         /* altered - modified */
 		m_time = nt2unixtime(getu64(fs, fname->mtime));
@@ -1745,7 +1960,7 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 		a_time = nt2unixtime(getu64(fs, fname->atime));
 
     	if (sec_skew != 0) {
-        	fprintf(hFile, "\nAdjusted $FILE_NAME Times:\n");
+        	fprintf(hFile, "\nAdjusted times:\n");
         	cr_time -= sec_skew;
         	m_time -= sec_skew;
         	a_time -= sec_skew;
@@ -1761,10 +1976,8 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
         	a_time += sec_skew;
         	c_time += sec_skew;
 
-        	fprintf(hFile, "\nOriginal $FILE_NAME Times:\n");
+        	fprintf(hFile, "\nOriginal times:\n");
 		}
-		else
-        	fprintf(hFile, "\n$FILE_NAME Times:\n");
 
 
     	fprintf(hFile, "Created:\t%s", ctime(&cr_time));
@@ -1772,14 +1985,106 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
     	fprintf(hFile, "MFT Modified:\t%s", ctime(&c_time));
     	fprintf(hFile, "Accessed:\t%s", ctime(&a_time));
 
-/*
-        printf ("fsize: %lu  Alloc size: %lu\n",
-		  getu64(fs, fname->real_fsize),
-		  getu64(fs, fname->alloc_fsize));
-*/
-
     }
 
+
+	/* $OBJECT_ID Information */
+	fs_data = fs_data_lookup(fs_inode->attr, NTFS_ATYPE_OBJID, 0);
+	if(fs_data) {
+        ntfs_attr_objid *objid = (ntfs_attr_objid *)fs_data->buf;
+		u_int64_t id1, id2;
+
+		fprintf(hFile, "\n$OBJECT_ID Attribute Values:\n");
+
+		id1 = getu64(fs, objid->objid1);
+		id2 = getu64(fs, objid->objid2);
+
+		fprintf(hFile, "Object Id: %.8"PRIx32"-%.4"PRIx16"-%.4"PRIx16"-%.4"PRIx16"-%.12"PRIx64"\n",
+		  (u_int32_t) (id2 >> 32) & 0xffffffff,
+		  (u_int16_t) (id2 >> 16) & 0xffff,
+		  (u_int16_t) (id2 & 0xffff),
+		  (u_int16_t) (id1 >> 48) & 0xffff,
+		  (u_int64_t) (id1 & (u_int64_t)0x0000ffffffffffffULL));
+
+
+		/* The rest of the  fields do not always exist.  Check the attr size */
+		if (fs_data->size > 16) {
+			id1 = getu64(fs, objid->orig_volid1);
+			id2 = getu64(fs, objid->orig_volid2);
+
+			fprintf(hFile, "Birth Volume Id: %.8"PRIx32"-%.4"PRIx16"-%.4"PRIx16"-%.4"PRIx16"-%.12"PRIx64"\n",
+			  (u_int32_t) (id2 >> 32) & 0xffffffff,
+			  (u_int16_t) (id2 >> 16) & 0xffff,
+			  (u_int16_t) (id2 & 0xffff),
+			  (u_int16_t) (id1 >> 48) & 0xffff,
+			  (u_int64_t)(id1 & (u_int64_t)0x0000ffffffffffffULL));
+		}
+
+		if (fs_data->size > 32) {
+			id1 = getu64(fs, objid->orig_objid1);
+			id2 = getu64(fs, objid->orig_objid2);
+
+			fprintf(hFile, "Birth Object Id: %.8"PRIx32"-%.4"PRIx16"-%.4"PRIx16"-%.4"PRIx16"-%.12"PRIx64"\n",
+			  (u_int32_t) (id2 >> 32) & 0xffffffff,
+			  (u_int16_t) (id2 >> 16) & 0xffff,
+			  (u_int16_t) (id2 & 0xffff),
+			  (u_int16_t) (id1 >> 48) & 0xffff,
+			  (u_int64_t)(id1 & (u_int64_t)0x0000ffffffffffffULL));
+		}
+
+		if (fs_data->size > 48) {
+			id1 = getu64(fs, objid->orig_domid1);
+			id2 = getu64(fs, objid->orig_domid2);
+
+			fprintf(hFile, "Birth Domain Id: %.8"PRIx32"-%.4"PRIx16"-%.4"PRIx16"-%.4"PRIx16"-%.12"PRIx64"\n",
+			  (u_int32_t) (id2 >> 32) & 0xffffffff,
+			  (u_int16_t) (id2 >> 16) & 0xffff,
+			  (u_int16_t) (id2 & 0xffff),
+			  (u_int16_t) (id1 >> 48) & 0xffff,
+			  (u_int64_t)(id1 & (u_int64_t)0x0000ffffffffffffULL));
+		}
+	}
+
+	/* Attribute List Information */
+	fs_data = fs_data_lookup(fs_inode->attr, NTFS_ATYPE_ATTRLIST, 0);
+	if(fs_data) {
+		char *buf;
+		ntfs_attrlist *list;
+		int endaddr;
+
+		fprintf(hFile, "\n$ATTRIBUTE_LIST Attribute Values:\n");
+
+		/* Get a copy of the attribute list stream using the above action*/
+		act_bufleft = (int)fs_data->size;
+		act_buf = buf = mymalloc (act_bufleft);
+		endaddr = (int)buf + fs_data->size;
+
+		ntfs_data_walk(ntfs, fs_data, 0,
+		  buffer_action, "attrlist");
+
+		/* this value should be zero, if not then we didn't read all of the
+		 * buffer
+		 */
+		if (act_bufleft > 0) {
+			fprintf(hFile, "error reading attribute list buffer\n");
+			goto egress;
+		}
+
+		/* Process the list & print the details */
+		for (list = (ntfs_attrlist *)buf;
+		  (list) && ((int)list < endaddr);
+		  list = (ntfs_attrlist *)((int)list + getu16(fs, list->len)) )
+		{
+			fprintf(hFile, "Type: %d-%d \tMFT Entry: %"PRIu64" \tVCN: %"PRIu64"\n",
+			  getu32(fs, list->type), getu16(fs, list->id),
+			  getu48(fs, list->file_ref),
+			  getu64(fs, list->start_vcn));
+		}
+egress:
+		free(buf);
+	}
+
+	/* Print all of the attributes */
     fs_data = fs_inode->attr;
 
     fprintf(hFile, "\nAttributes: \n");
@@ -1788,15 +2093,20 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
     
         ntfs_attrname_lookup(fs, fs_data->type, type, 256);
 
-        printf("Type: %s (%i-%i)   Name: %s   %sResident   size: %llu\n",
+        printf("Type: %s (%i-%i)   Name: %s   %sResident%s%s%s   size: %llu\n",
           type, fs_data->type, fs_data->id, fs_data->name,
-          (fs_data->flags & FS_DATA_NONRES)?"Non-":"", (ULLONG)fs_data->size);
-    
+          (fs_data->flags & FS_DATA_NONRES)?"Non-":"", 
+          (fs_data->flags & FS_DATA_ENC)?", Encrypted":"", 
+          (fs_data->flags & FS_DATA_COMP)?", Compressed":"", 
+          (fs_data->flags & FS_DATA_SPAR)?", Sparse":"", 
+		  (ULLONG)fs_data->size);
+
+   		/* print the layout if it is non-resident and not "special" */
         if (fs_data->flags & FS_DATA_NONRES) {
             printidx = 0;
 
             fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id, 
-			   (FS_FLAG_FILE_AONLY | FS_FLAG_FILE_SLACK),
+			   (FS_FLAG_FILE_AONLY | FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOABORT),
                print_addr_act, (char *)hFile);
             if (printidx != 0)
                 printf("\n");
@@ -1850,8 +2160,12 @@ ntfs_open(const char *name, unsigned char ftype)
 		error("%s: read superblock: %m", name);
 
 	/* Check the magic value */
-	if (guessu32(fs, ntfs->fs->magic, NTFS_FS_MAGIC)) 
-           error("Error: %s is not a NTFS file system", name);
+	//if (guessu32(fs, ntfs->fs->magic, NTFS_FS_MAGIC)) 
+	if (guessu16(fs, ntfs->fs->magic, NTFS_FS_MAGIC)) 
+		error("Error: %s is not a NTFS file system", name);
+
+	//if (getu16(fs, ntfs->fs->magic2) != NTFS_FS_MAGIC2) 
+	//	error("Error: Invalid Magic 2 in NTFS boot sector");
 
 	/*
      * block calculations : although there are no blocks in ntfs,
