@@ -124,7 +124,7 @@ is_time(u_int64_t t)
 /* 
  * Process a list of directory entries, starting at idxe with a length
  * of _size_ bytes.  _len_ is the length that is reported in the 
- * idxbuf header.  in other words, everything after _len_ bytes is
+ * idxelist header.  in other words, everything after _len_ bytes is
  * considered unallocated area and therefore deleted content 
  *
  * The only flag that we care about is ALLOC (no UNALLOC entries exist
@@ -156,12 +156,12 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 
 		if (verbose)
 			fprintf(logfp,
-			  "ntfs_dent_idxentry: Fname Entry: %lu  File Ref: %lu  IdxEnt Len: %lu  StrLen: %lu\n",
-			  (ULONG)fname, (ULONG)getu48(fs, idxe->file_ref), 
+			  "ntfs_dent_idxentry: Idxe: %lu $FILE_NAME Entry: %lu  File Ref: %lu  IdxEnt Len: %lu  StrLen: %lu\n",
+			  (ULONG)idxe, (ULONG)fname, (ULONG)getu48(fs, idxe->file_ref), 
 			  (ULONG)getu16(fs, idxe->idxlen), (ULONG)getu16(fs, idxe->strlen));
 
-		/* perform some sanity checks & advance by 4-bytes if 
-		 * this is not a valid entry
+		/* perform some sanity checks on index buffer head
+		 * and advance by 4-bytes if invalid
 		 */
 		if ((getu48(fs, idxe->file_ref) > fs->last_inum) || 
 		   (getu48(fs, idxe->file_ref) < fs->first_inum) ||
@@ -173,28 +173,36 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 			continue;
 		}
 
-		/* a deleted entry will have strlen of 0, so do some sanity
-		 * checks first before we skip it.
+		/* do some sanity checks on the deleted entries
 		 */
 		if  ( (getu16(fs, idxe->strlen) == 0) ||
 		  (((int)idxe + getu16(fs, idxe->idxlen)) > endaddr_alloc)) {
 
-			if (
-			 ((fname->nspace != NTFS_FNAME_POSIX) &&
+			/* name space checks */
+			if ((fname->nspace != NTFS_FNAME_POSIX) &&
 			  (fname->nspace != NTFS_FNAME_WIN32) &&
 			  (fname->nspace != NTFS_FNAME_DOS) &&
-			  (fname->nspace != NTFS_FNAME_WINDOS) ) ||  
-		
+			  (fname->nspace != NTFS_FNAME_WINDOS) ) {
+				idxe = (ntfs_idxentry *)((int)idxe + 4);
+				continue;
+			}
+
+			if (
 			  ((int)getu64(fs, fname->alloc_fsize) < (int)getu64(fs, fname->real_fsize)) ||
-			  (getu64(fs, fname->real_fsize) < fname->nlen) ||
 			  (fname->nlen == 0) ||
-			  (*(u_int8_t *)&fname->name == 0)  ||
-			  (is_time(getu64(fs, fname->crtime)) == 0) ||
+			  (*(u_int8_t *)&fname->name == 0)) {
+
+				idxe = (ntfs_idxentry *)((int)idxe + 4);
+				continue;
+			}
+
+			if ((is_time(getu64(fs, fname->crtime)) == 0) ||
 			  (is_time(getu64(fs, fname->atime)) == 0) ||
 			  (is_time(getu64(fs, fname->mtime)) == 0) ) { 
 
 				idxe = (ntfs_idxentry *)((int)idxe + 4);
 				continue;
+
 			}
 		}
 		
@@ -204,7 +212,12 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 		 * full POSIX or WIN32 that does not satisfy DOS, then a 
 		 * type NTFS_FNAME_DOS will exist.  If the name is WIN32,
 		 * but already satisfies DOS, then a type NTFS_FNAME_WINDOS
-		 * will exist */
+		 * will exist 
+		 *
+		 * Note that we could be missing some info from deleted files
+		 * if the windows version was deleted and the DOS wasn't...
+		 */
+
 		if (fname->nspace == NTFS_FNAME_DOS)  {
 			if (verbose)
 				fprintf(logfp,
@@ -229,12 +242,12 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 		 * Check if this entry is deleted
 		 *
 		 * The final check is to see if the end of this entry is 
-		 * within the space that the idxbuf claimed was valid
+		 * within the space that the idxallocbuf claimed was valid
 		 */
 		if ((getu16(fs, idxe->strlen) == 0) || 
 		  (((int)idxe + getu16(fs, idxe->idxlen)) > endaddr_alloc)) {
 
-			/* we know entries with an inode of 0 are not legit because
+			/* we know deleted entries with an inode of 0 are not legit because
 			 * that is the MFT value.  Free it so it does not confuse
 			 * people with invalid data
 			 */
@@ -278,13 +291,8 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 		} /* end of recurse */
 
 
-		/* We could check for the NTFS_IDX_LAST flag here, but
-		 * I think we may lose some deleted file entries by 
-		 * doing that
-		 */
 incr_entry:
 
-
 		/* the theory here is that deleted entries have strlen == 0 and
 		 * have been found to have idxlen == 16
 		 *
@@ -318,7 +326,7 @@ incr_entry:
  *
  */
 static void
-ntfs_fix_idxbuf(NTFS_INFO *ntfs, ntfs_idxbuf *idxbuf, u_int32_t len)
+ntfs_fix_idxrec(NTFS_INFO *ntfs, ntfs_idxrec *idxrec, u_int32_t len)
 {
 	int i;
 	u_int16_t	orig_seq;
@@ -327,28 +335,28 @@ ntfs_fix_idxbuf(NTFS_INFO *ntfs, ntfs_idxbuf *idxbuf, u_int32_t len)
 
 	if (verbose)
 		fprintf(logfp,
-		  "ntfs_fix_idxbuf: Fixing idxbuf: %lu  Len: %lu\n",
-		  (ULONG)idxbuf, (ULONG)len);
+		  "ntfs_fix_idxrec: Fixing idxrec: %lu  Len: %lu\n",
+		  (ULONG)idxrec, (ULONG)len);
 
     /* sanity check so we don't run over in the next loop */
-    if ((getu16(fs, idxbuf->upd_cnt) - 1) * ntfs->ssize_b > len)
-        error ("More Update Sequence Entries than idx buf size");
+    if ((getu16(fs, idxrec->upd_cnt) - 1) * ntfs->ssize_b > len)
+        error ("More Update Sequence Entries than idx record size");
 
     /* Apply the update sequence structure template */
-    upd = (ntfs_upd *)((int)idxbuf + getu16(fs, idxbuf->upd_off));
+    upd = (ntfs_upd *)((int)idxrec + getu16(fs, idxrec->upd_off));
    
     /* Get the sequence value that each 16-bit value should be */
     orig_seq = getu16(fs, upd->upd_val);
 
     /* cycle through each sector */
-    for (i = 1; i < getu16(fs, idxbuf->upd_cnt); i++) {
+    for (i = 1; i < getu16(fs, idxrec->upd_cnt); i++) {
 
         /* The offset into the buffer of the value to analyze */
 		int offset = i * ntfs->ssize_b - 2;
 		u_int8_t *new, *old;
 
         /* get the current sequence value */
-		u_int16_t cur_seq = getu16(fs, (int)idxbuf + offset);
+		u_int16_t cur_seq = getu16(fs, (int)idxrec + offset);
 
 		if (cur_seq != orig_seq) {
 			/* get the replacement value */
@@ -359,11 +367,11 @@ ntfs_fix_idxbuf(NTFS_INFO *ntfs, ntfs_idxbuf *idxbuf, u_int32_t len)
 		}
 
 		new = &upd->upd_seq + (i-1) * 2;
-		old = (u_int8_t *)(int)idxbuf + offset;
+		old = (u_int8_t *)(int)idxrec + offset;
 
         if (verbose)
 			fprintf(logfp,
-			  "ntfs_fix_idxbuf: upd_seq %i   Replacing: %.4x   With: %.4x\n",
+			  "ntfs_fix_idxrec: upd_seq %i   Replacing: %.4x   With: %.4x\n",
 			  i, getu16(fs, old), getu16(fs, new));
 
 		*old++ = *new++;	
@@ -391,7 +399,8 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 	char 			*idxalloc;
 	ntfs_idxentry 	*idxe;
 	ntfs_idxroot 	*idxroot;
-	ntfs_idxbuf 	*idxbuf_p, *idxbuf;
+	ntfs_idxelist 	*idxelist;
+	ntfs_idxrec 	*idxrec_p, *idxrec;
 	int off, idxalloc_len;
 
 	/* sanity check */
@@ -491,19 +500,25 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 		error ("$IDX_ROOT is not resident - it should be");
 
 	idxroot = (ntfs_idxroot *)fs_data_root->buf;
-	idxe = (ntfs_idxentry *)&idxroot->stream;
 
-	/* Get the type that is used to sort the tree */
-	/* @@@ We should handle more things besides FNAME */
+	/* Verify that the attribute type is $FILE_NAME */
 	if (getu32(fs, idxroot->type) == 0)
 		return;
 	else if (getu32(fs, idxroot->type) != NTFS_ATYPE_FNAME)
 		error ("ERROR: Directory index is sorted by type: %d.\nOnly $FNAME is currently supported", getu32(fs, idxroot->type));
 
-	/* Process it */
-	ntfs_dent_idxentry (ntfs, idxe, getu32(fs, idxroot->alloc),
-	  getu32(fs, idxroot->strlen), flags, action, ptr);
+	/* Get the header of the index entry list */
+	idxelist = &idxroot->list;
 
+	/* Get the offset to the start of the index entry list */
+	idxe = (ntfs_idxentry *)((int)idxelist +
+		  getu32(fs, idxelist->begin_off));
+
+	/* Process $IDX_ROOT  */
+	ntfs_dent_idxentry (ntfs, idxe, 
+	  getu32(fs, idxelist->buf_off) - getu32(fs, idxelist->begin_off),
+	  getu32(fs, idxelist->end_off) - getu32(fs, idxelist->begin_off), 
+	  flags, action, ptr);
 
 	/* 
 	 * get the index allocation attribute if it exists (it doesn't for 
@@ -516,8 +531,8 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 	 * all of the entries 
 	 */
 	if (!fs_data_alloc) {
-		if (idxroot->flags & NTFS_IDX_LARGE)
-			error("Error: $IDX_ROOT says there should be an $IDX_ALLOC, but there isn't");
+		if (getu32(fs, idxelist->flags) & NTFS_IDXELIST_CHILD)
+			error("Error: $IDX_ROOT says there should be children, but there isn't");
 
 		return;
 	}
@@ -535,9 +550,8 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 	if (verbose)
 		fprintf (logfp, "ntfs_dent_walk: Copying $IDX_ALLOC into buffer\n");
 
-	fs->file_walk(fs, fs_inode, NTFS_ATYPE_IDXALLOC, 0, 
-		FS_FLAG_FILE_SLACK,
-		idxalloc_action, "index alloc");
+	ntfs_data_walk(ntfs, fs_data_alloc,
+	   FS_FLAG_FILE_SLACK, idxalloc_action, "index alloc");
 
 	if (bufleft > 0)
 		error ("ntfs_dent_walk: Buffer not filled when copying $IDX_ALLOC (%d)",
@@ -546,101 +560,107 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 
 	/*
 	 * The idxalloc is a big buffer that contains one or more
-	 * idx buffer structures.  Each idxbuf is a sub-node in the B-Tree.  
+	 * idx buffer structures.  Each idxrec is a node in the B-Tree.  
 	 * We do not process the tree as a tree because then we could
 	 * not find the deleted file names.
 	 *
-	 * Therefore, we scan the big buffer looking for the idxbuf
-	 * structures.  We save a pointer to the known beginning (idxbuf_p).
-	 * Then we scan for the beginning of the next one (idxbuf) and process
-	 * everything in the middle as an idxbuf.  We can't use the
+	 * Therefore, we scan the big buffer looking for the index record
+	 * structures.  We save a pointer to the known beginning (idxrec_p).
+	 * Then we scan for the beginning of the next one (idxrec) and process
+	 * everything in the middle as an ntfs_idxrec.  We can't use the
 	 * size given because then we wouldn't see the deleted names
 	 */
 
 	/* Set the previous pointer to NULL */
-	idxbuf_p = idxbuf = NULL;
+	idxrec_p = idxrec = NULL;
 
 	/* Loop by cluster size */
 	for (off = 0; off < idxalloc_len; off += ntfs->csize_b) {
-		int idx_len, buf_len;
-		idxbuf = (ntfs_idxbuf *)&idxalloc[off];
+		int list_len, rec_len;
+
+		idxrec = (ntfs_idxrec *)&idxalloc[off];
 
 		if (verbose)
 			fprintf (logfp, 
 			  "ntfs_dent_walk: Index Buffer Offset: %d  Magic: %x\n",
-			  off, getu32(fs, idxbuf->magic));
+			  off, getu32(fs, idxrec->magic));
 
-		/* Is this the begining of an index buffer? */
-		if (getu32(fs, idxbuf->magic) != NTFS_IDXBUF_MAGIC) 
+		/* Is this the begining of an index record? */
+		if (getu32(fs, idxrec->magic) != NTFS_IDXREC_MAGIC) 
 			continue;
 
 
-		/* idxbuf_p is only NULL for the first time */
-		if (idxbuf_p == NULL) {
-			idxbuf_p = idxbuf;
+		/* idxrec_p is only NULL for the first time 
+		 * Set it and start again to find the next one */
+		if (idxrec_p == NULL) {
+			idxrec_p = idxrec;
 			continue;
 		}
 
 		/* Process the previous structure */
 
-		/* idxbuf points to the next idxbuf structure, idxbuf_p
+		/* idxrec points to the next idxrec structure, idxrec_p
 		 * points to the one we are going to process
 		 */
-		buf_len = ((int)idxbuf - (int)idxbuf_p);
+		rec_len = ((int)idxrec - (int)idxrec_p);
 
 		if (verbose)
 			fprintf (logfp, 
-			  "ntfs_dent_walk: Processing previous index buffer (len: %lu)\n",
-			  (ULONG)buf_len);
+			  "ntfs_dent_walk: Processing previous index record (len: %lu)\n",
+			  (ULONG)rec_len);
 
-		/* remove the update sequence */
-		ntfs_fix_idxbuf(ntfs, idxbuf_p, buf_len);
+		/* remove the update sequence in the index record */
+		ntfs_fix_idxrec(ntfs, idxrec_p, rec_len);
 
-		/* I'm not sure why it is off by 0x18, but that is what the
-		 * sourceforge docs say and it works 
+		/* Locate the start of the index entry list */
+		idxelist = &idxrec_p->list;
+		idxe = (ntfs_idxentry *)((int)idxelist +
+		  getu32(fs, idxelist->begin_off));
+
+		/* the length from the start of the next record to where our
+		 * list starts.
+		 * This should be the same as buf_off in idxelist, but we don't
+		 * trust it.
 		 */
-		idxe = (ntfs_idxentry *)((int)idxbuf_p +
-		  getu16(fs, idxbuf_p->begin_off) + 0x18);
-
-		/* This is the length of the idx entries */
-		idx_len = (int)idxbuf - (int)idxe;
+		list_len = (int)idxrec - (int)idxe;
 
 		/* process the list of index entries */
-		ntfs_dent_idxentry (ntfs, idxe, idx_len, 
-		  getu32(fs, idxbuf_p->end_off) - getu16(fs, idxbuf_p->begin_off),
+		ntfs_dent_idxentry (ntfs, idxe, 
+		  list_len,
+		  getu32(fs, idxelist->end_off) - getu32(fs, idxelist->begin_off),
 		  flags, action, ptr);
 
-		/* reset the pointer to the next buffer */
-		idxbuf_p = idxbuf;
+		/* reset the pointer to the next record */
+		idxrec_p = idxrec;
 
 	} /* end of cluster loop */
  
 
-	/* Process the final buffer */
-	if (idxbuf_p) {
-		int idx_len, buf_len;
-		buf_len = idxalloc_len - ((int)idxbuf_p - (int)idxalloc);
+	/* Process the final record */
+	if (idxrec_p) {
+		int list_len, rec_len;
+
+		/* Length from end of attribute to start of this */
+		rec_len = idxalloc_len - ((int)idxrec_p - (int)idxalloc);
 
 		if (verbose)
 			fprintf (logfp, 
-			  "ntfs_dent_walk: Processing final index buffer (len: %lu)\n",
-			  (ULONG)buf_len);
+			  "ntfs_dent_walk: Processing final index record (len: %lu)\n",
+			  (ULONG)rec_len);
 
 		/* remove the update sequence */
-		ntfs_fix_idxbuf(ntfs, idxbuf_p, buf_len);
+		ntfs_fix_idxrec(ntfs, idxrec_p, rec_len);
 
-		/* I'm not sure why it is off by 0x18, but that is what the
-		 * sourceforge docs say and it works 
-		 */
-		idxe = (ntfs_idxentry *)((int)idxbuf_p +
-		  getu16(fs, idxbuf_p->begin_off) + 0x18);
+		idxelist = &idxrec_p->list;
+		idxe = (ntfs_idxentry *)((int)idxelist + 
+		  getu32(fs, idxelist->begin_off));
 
 		/* This is the length of the idx entries */
-		idx_len = buf_len - (getu16(fs, idxbuf_p->begin_off) + 0x18);
+		list_len = rec_len - getu32(fs, idxelist->begin_off);
 
 		/* process the list of index entries */
-		ntfs_dent_idxentry (ntfs, idxe, idx_len, 
-		  getu32(fs, idxbuf_p->end_off) - getu16(fs, idxbuf_p->begin_off),
+		ntfs_dent_idxentry (ntfs, idxe, list_len, 
+		  getu32(fs, idxelist->end_off) - getu32(fs, idxelist->begin_off),
 		  flags, action, ptr);
 	}
 
