@@ -219,12 +219,15 @@ extern MM_INFO *mac_open(const char *, unsigned char ftype, daddr_t);
 extern MM_INFO *bsd_open(const char *, unsigned char ftype, daddr_t);
 
 extern MM_INFO *sun_open(const char *, unsigned char ftype, daddr_t);
+
+extern MM_INFO *gpt_open(const char *, unsigned char ftype, daddr_t);
 /* 
  * Endian Ordering 
  */
 extern u_int8_t guessu16(MM_INFO *, u_int8_t *, u_int16_t);
 extern u_int8_t guessu32(MM_INFO *, u_int8_t *, u_int32_t);
 
+
 #define getu16(mm, x)   \
     ((mm->flags & MM_LIT_ENDIAN) ? \
       (((u_int8_t *)x)[0] + (((u_int8_t *)x)[1] << 8)) :    \
@@ -243,4 +246,33 @@ extern u_int8_t guessu32(MM_INFO *, u_int8_t *, u_int32_t);
       (((u_int8_t *)x)[1] << 16) + \
       (((u_int8_t *)x)[0] << 24) ) )
 
+
+
+#define getu64(fs, x)   \
+	(u_int64_t)( (fs->flags & MM_LIT_ENDIAN)  ?     \
+	((u_int64_t) \
+	((u_int64_t)((u_int8_t *)(x))[0] << 0)  + \
+	((u_int64_t)((u_int8_t *)(x))[1] << 8) + \
+	((u_int64_t)((u_int8_t *)(x))[2] << 16) + \
+	((u_int64_t)((u_int8_t *)(x))[3] << 24) + \
+	((u_int64_t)((u_int8_t *)(x))[4] << 32) + \
+	((u_int64_t)((u_int8_t *)(x))[5] << 40) + \
+	((u_int64_t)((u_int8_t *)(x))[6] << 48) + \
+        ((u_int64_t)((u_int8_t *)(x))[7] << 56)) \
+	: \
+	((u_int64_t) \
+	((u_int64_t)((u_int8_t *)(x))[7] <<  0) + \
+	((u_int64_t)((u_int8_t *)(x))[6] <<  8) + \
+	((u_int64_t)((u_int8_t *)(x))[5] << 16) + \
+	((u_int64_t)((u_int8_t *)(x))[4] << 24) + \
+	((u_int64_t)((u_int8_t *)(x))[3] << 32) + \
+	((u_int64_t)((u_int8_t *)(x))[2] << 40) + \
+	((u_int64_t)((u_int8_t *)(x))[1] << 48) + \
+	((u_int64_t)((u_int8_t *)(x))[0] << 56)) )
+
+
+
+extern void
+uni2ascii(u_int8_t *, int , u_int8_t *, int );
+
 #endif
